@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -12,6 +12,7 @@
 #endif
 #include <IceUtil/DisableWarnings.h>
 #include <Communicator.h>
+#include <ImplicitContext.h>
 #include <Logger.h>
 #include <ObjectAdapter.h>
 #include <ObjectFactory.h>
@@ -64,9 +65,9 @@ static CommunicatorObject*
 communicatorNew(PyObject* /*arg*/)
 {
     CommunicatorObject* self = PyObject_New(CommunicatorObject, &CommunicatorType);
-    if (self == NULL)
+    if(!self)
     {
-        return NULL;
+        return 0;
     }
     self->communicator = 0;
     self->wrapper = 0;
@@ -82,16 +83,49 @@ extern "C"
 static int
 communicatorInit(CommunicatorObject* self, PyObject* args, PyObject* /*kwds*/)
 {
-    PyObject* arglist = NULL;
-    PyObject* initDataType = lookupType("Ice.InitializationData");
-    PyObject* initData = NULL;
-    if(!PyArg_ParseTuple(args, STRCAST("|O!O!"), &PyList_Type, &arglist, initDataType, &initData))
+    PyObject* argList = 0;
+    PyObject* initData = 0;
+    if(!PyArg_ParseTuple(args, STRCAST("|OO"), &argList, &initData))
     {
         return -1;
     }
 
+    if(argList == Py_None)
+    {
+        argList = 0;
+    }
+
+    if(initData == Py_None)
+    {
+        initData = 0;
+    }
+
+    PyObject* initDataType = lookupType("Ice.InitializationData");
+
+    if(argList && !initData)
+    {
+        if(PyObject_IsInstance(argList, initDataType))
+        {
+            initData = argList;
+            argList = 0;
+        }
+        else if(!PyList_Check(argList))
+        {
+            PyErr_Format(PyExc_ValueError, STRCAST("initialize expects an argument list or Ice.InitializationData"));
+            return -1;
+        }
+    }
+    else if(argList && initData)
+    {
+        if(!PyList_Check(argList) || !PyObject_IsInstance(initData, initDataType))
+        {
+            PyErr_Format(PyExc_ValueError, STRCAST("initialize expects an argument list and Ice.InitializationData"));
+            return -1;
+        }
+    }
+
     Ice::StringSeq seq;
-    if(arglist && !listToStringSeq(arglist, seq))
+    if(argList && !listToStringSeq(argList, seq))
     {
         return -1;
     }
@@ -104,38 +138,29 @@ communicatorInit(CommunicatorObject* self, PyObject* args, PyObject* /*kwds*/)
     Ice::InitializationData data;
     if(initData)
     {
-	PyObjectHandle properties = PyObject_GetAttrString(initData, STRCAST("properties"));
-	PyObjectHandle logger = PyObject_GetAttrString(initData, STRCAST("logger"));
-	PyObjectHandle defaultContext = PyObject_GetAttrString(initData, STRCAST("defaultContext"));
-	PyObjectHandle threadHook = PyObject_GetAttrString(initData, STRCAST("threadHook"));
+        PyObjectHandle properties = PyObject_GetAttrString(initData, STRCAST("properties"));
+        PyObjectHandle logger = PyObject_GetAttrString(initData, STRCAST("logger"));
+        PyObjectHandle threadHook = PyObject_GetAttrString(initData, STRCAST("threadHook"));
 
-	if(properties.get() && properties.get() != Py_None)
-	{
-	    //
-	    // Get the properties implementation.
-	    //
-	    PyObjectHandle impl = PyObject_GetAttrString(properties.get(), STRCAST("_impl"));
-	    assert(impl.get() != NULL);
-	    data.properties = getProperties(impl.get());
-	}
+        if(properties.get() && properties.get() != Py_None)
+        {
+            //
+            // Get the properties implementation.
+            //
+            PyObjectHandle impl = PyObject_GetAttrString(properties.get(), STRCAST("_impl"));
+            assert(impl.get());
+            data.properties = getProperties(impl.get());
+        }
 
-	if(logger.get() && logger.get() != Py_None)
-	{
-	    data.logger = new LoggerWrapper(logger.get());
-	}
+        if(logger.get() && logger.get() != Py_None)
+        {
+            data.logger = new LoggerWrapper(logger.get());
+        }
 
-	if(defaultContext.get() && defaultContext.get() != Py_None)
-	{
-	    if(!dictionaryToContext(defaultContext.get(), data.defaultContext))
-	    {
-		return -1;
-	    }
-	}
-
-	if(threadHook.get() && threadHook.get() != Py_None)
-	{
-	    data.threadHook = new ThreadNotificationWrapper(threadHook.get());
-	}
+        if(threadHook.get() && threadHook.get() != Py_None)
+        {
+            data.threadHook = new ThreadNotificationWrapper(threadHook.get());
+        }
 
     }
 
@@ -157,51 +182,51 @@ communicatorInit(CommunicatorObject* self, PyObject* args, PyObject* /*kwds*/)
     int i = 0;
     for(Ice::StringSeq::const_iterator s = seq.begin(); s != seq.end(); ++s, ++i)
     {
-	argv[i] = strdup(s->c_str());
+        argv[i] = strdup(s->c_str());
     }
     argv[argc] = 0;
 
     Ice::CommunicatorPtr communicator;
     try
     {
-	if(hasArgs)
-	{
-	    communicator = Ice::initialize(argc, argv, data);
-	}
-	else
-	{
-	    communicator = Ice::initialize(data);
-	}
+        if(hasArgs)
+        {
+            communicator = Ice::initialize(argc, argv, data);
+        }
+        else
+        {
+            communicator = Ice::initialize(data);
+        }
     }
     catch(const Ice::Exception& ex)
     {
-	for(i = 0; i < argc + 1; ++i)
-	{
-	    free(argv[i]);
-	}
-	delete[] argv;
+        for(i = 0; i < argc + 1; ++i)
+        {
+            free(argv[i]);
+        }
+        delete[] argv;
 
-	setPythonException(ex);
-	return -1;
+        setPythonException(ex);
+        return -1;
     }
     
     //
     // Replace the contents of the given argument list with the filtered arguments.
     //
-    if(arglist)
+    if(argList)
     {
-	PyList_SetSlice(arglist, 0, PyList_Size(arglist), NULL); // Clear the list.
+        PyList_SetSlice(argList, 0, PyList_Size(argList), 0); // Clear the list.
 
-	for(i = 0; i < argc; ++i)
-	{
-	    PyObjectHandle str = Py_BuildValue(STRCAST("s"), argv[i]);
-	    PyList_Append(arglist, str.get());
-	}
+        for(i = 0; i < argc; ++i)
+        {
+            PyObjectHandle str = Py_BuildValue(STRCAST("s"), argv[i]);
+            PyList_Append(argList, str.get());
+        }
     }
 
     for(i = 0; i < argc + 1; ++i)
     {
-	free(argv[i]);
+        free(argv[i]);
     }
     delete[] argv;
 
@@ -212,9 +237,9 @@ communicatorInit(CommunicatorObject* self, PyObject* args, PyObject* /*kwds*/)
     CommunicatorMap::iterator p = _communicatorMap.find(communicator);
     if(p != _communicatorMap.end())
     {
-	_communicatorMap.erase(p);
+        _communicatorMap.erase(p);
     }
-    _communicatorMap.insert(CommunicatorMap::value_type(communicator, (PyObject*)self));
+    _communicatorMap.insert(CommunicatorMap::value_type(communicator, reinterpret_cast<PyObject*>(self)));
 
     return 0;
 }
@@ -225,13 +250,16 @@ extern "C"
 static void
 communicatorDealloc(CommunicatorObject* self)
 {
-    CommunicatorMap::iterator p = _communicatorMap.find(*self->communicator);
-    //
-    // find() can fail if an error occurred during communicator initialization.
-    //
-    if(p != _communicatorMap.end())
+    if(self->communicator)
     {
-	_communicatorMap.erase(p);
+        CommunicatorMap::iterator p = _communicatorMap.find(*self->communicator);
+        //
+        // find() can fail if an error occurred during communicator initialization.
+        //
+        if(p != _communicatorMap.end())
+        {
+            _communicatorMap.erase(p);
+        }
     }
 
     if(self->shutdownThread)
@@ -254,13 +282,13 @@ communicatorDestroy(CommunicatorObject* self)
     assert(self->communicator);
     try
     {
-	AllowThreads allowThreads; // Release Python's global interpreter lock to avoid a potential deadlock.
+        AllowThreads allowThreads; // Release Python's global interpreter lock to avoid a potential deadlock.
         (*self->communicator)->destroy();
     }
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     Py_INCREF(Py_None);
@@ -276,13 +304,13 @@ communicatorShutdown(CommunicatorObject* self)
     assert(self->communicator);
     try
     {
-	AllowThreads allowThreads; // Release Python's global interpreter lock to avoid a potential deadlock.
+        AllowThreads allowThreads; // Release Python's global interpreter lock to avoid a potential deadlock.
         (*self->communicator)->shutdown();
     }
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     Py_INCREF(Py_None);
@@ -307,7 +335,7 @@ communicatorWaitForShutdown(CommunicatorObject* self, PyObject* args)
     int timeout = 0;
     if(!PyArg_ParseTuple(args, STRCAST("i"), &timeout))
     {
-        return NULL;
+        return 0;
     }
 
     assert(timeout > 0);
@@ -332,20 +360,19 @@ communicatorWaitForShutdown(CommunicatorObject* self, PyObject* args)
                 t->start();
             }
 
-	    while(!self->shutdown)
-	    {
-		bool done;
-		{
-		    AllowThreads allowThreads; // Release Python's global interpreter lock during blocking calls.
-		    done = (*self->shutdownMonitor).timedWait(IceUtil::Time::milliSeconds(timeout));
-		}
-		
-		if(!done)
-		{
-		    Py_INCREF(Py_False);
-		    return Py_False;
-		}
-	    }
+            while(!self->shutdown)
+            {
+                bool done;
+                {
+                    AllowThreads allowThreads; // Release Python's global interpreter lock during blocking calls.
+                    done = (*self->shutdownMonitor).timedWait(IceUtil::Time::milliSeconds(timeout));
+                }
+                
+                if(!done)
+                {
+                    PyRETURN_FALSE;
+                }
+            }
         }
 
         assert(self->shutdown);
@@ -354,7 +381,7 @@ communicatorWaitForShutdown(CommunicatorObject* self, PyObject* args)
         if(ex)
         {
             setPythonException(*ex);
-            return NULL;
+            return 0;
         }
     }
     else
@@ -367,12 +394,39 @@ communicatorWaitForShutdown(CommunicatorObject* self, PyObject* args)
         catch(const Ice::Exception& ex)
         {
             setPythonException(ex);
-            return NULL;
+            return 0;
         }
     }
 
-    Py_INCREF(Py_True);
-    return Py_True;
+    PyRETURN_TRUE;
+}
+
+#ifdef WIN32
+extern "C"
+#endif
+static PyObject*
+communicatorIsShutdown(CommunicatorObject* self)
+{
+    assert(self->communicator);
+    bool isShutdown;
+    try
+    {
+        isShutdown = (*self->communicator)->isShutdown();
+    }
+    catch(const Ice::Exception& ex)
+    {
+        setPythonException(ex);
+        return 0;
+    }
+
+    if(isShutdown)
+    {
+        PyRETURN_TRUE;
+    }
+    else
+    {
+        PyRETURN_FALSE;
+    }
 }
 
 #ifdef WIN32
@@ -384,7 +438,7 @@ communicatorStringToProxy(CommunicatorObject* self, PyObject* args)
     char* str;
     if(!PyArg_ParseTuple(args, STRCAST("s"), &str))
     {
-        return NULL;
+        return 0;
     }
 
     assert(self->communicator);
@@ -396,7 +450,7 @@ communicatorStringToProxy(CommunicatorObject* self, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     return createProxy(proxy, *self->communicator);
@@ -411,7 +465,7 @@ communicatorProxyToString(CommunicatorObject* self, PyObject* args)
     PyObject* obj;
     if(!PyArg_ParseTuple(args, STRCAST("O!"), &ProxyType, &obj))
     {
-        return NULL;
+        return 0;
     }
 
     Ice::ObjectPrx proxy = getProxy(obj);
@@ -425,12 +479,39 @@ communicatorProxyToString(CommunicatorObject* self, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     return PyString_FromString(const_cast<char*>(str.c_str()));
 }
 
+#ifdef WIN32
+extern "C"
+#endif
+static PyObject*
+communicatorPropertyToProxy(CommunicatorObject* self, PyObject* args)
+{
+    char* str;
+    if(!PyArg_ParseTuple(args, STRCAST("s"), &str))
+    {
+        return 0;
+    }
+
+    assert(self->communicator);
+    Ice::ObjectPrx proxy;
+    try
+    {
+        proxy = (*self->communicator)->propertyToProxy(str);
+    }
+    catch(const Ice::Exception& ex)
+    {
+        setPythonException(ex);
+        return 0;
+    }
+
+    return createProxy(proxy, *self->communicator);
+}
+
 #ifdef WIN32
 extern "C"
 #endif
@@ -440,7 +521,7 @@ communicatorStringToIdentity(CommunicatorObject* self, PyObject* args)
     char* str;
     if(!PyArg_ParseTuple(args, STRCAST("s"), &str))
     {
-        return NULL;
+        return 0;
     }
 
     assert(self->communicator);
@@ -452,7 +533,7 @@ communicatorStringToIdentity(CommunicatorObject* self, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     return createIdentity(id);
@@ -468,13 +549,13 @@ communicatorIdentityToString(CommunicatorObject* self, PyObject* args)
     PyObject* obj;
     if(!PyArg_ParseTuple(args, STRCAST("O!"), identityType, &obj))
     {
-        return NULL;
+        return 0;
     }
 
     Ice::Identity id;
     if(!getIdentity(obj, id))
     {
-        return NULL;
+        return 0;
     }
     string str;
 
@@ -486,7 +567,7 @@ communicatorIdentityToString(CommunicatorObject* self, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     return PyString_FromString(const_cast<char*>(str.c_str()));
@@ -501,13 +582,13 @@ communicatorFlushBatchRequests(CommunicatorObject* self)
     assert(self->communicator);
     try
     {
-	AllowThreads allowThreads; // Release Python's global interpreter lock to avoid a potential deadlock.
+        AllowThreads allowThreads; // Release Python's global interpreter lock to avoid a potential deadlock.
         (*self->communicator)->flushBatchRequests();
     }
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     Py_INCREF(Py_None);
@@ -523,10 +604,10 @@ communicatorSetWrapper(CommunicatorObject* self, PyObject* args)
     PyObject* wrapper;
     if(!PyArg_ParseTuple(args, STRCAST("O"), &wrapper))
     {
-        return NULL;
+        return 0;
     }
 
-    assert(self->wrapper == NULL);
+    assert(!self->wrapper);
     self->wrapper = wrapper;
     Py_INCREF(self->wrapper);
 
@@ -540,7 +621,7 @@ extern "C"
 static PyObject*
 communicatorGetWrapper(CommunicatorObject* self)
 {
-    assert(self->wrapper != NULL);
+    assert(self->wrapper);
     Py_INCREF(self->wrapper);
     return self->wrapper;
 }
@@ -560,7 +641,7 @@ communicatorGetProperties(CommunicatorObject* self)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     return createProperties(properties);
@@ -581,7 +662,7 @@ communicatorGetLogger(CommunicatorObject* self)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     //
@@ -594,9 +675,9 @@ communicatorGetLogger(CommunicatorObject* self)
     LoggerWrapperPtr wrapper = LoggerWrapperPtr::dynamicCast(logger);
     if(wrapper)
     {
-	PyObject* obj = wrapper->getObject();
-	Py_INCREF(obj);
-	return obj;
+        PyObject* obj = wrapper->getObject();
+        Py_INCREF(obj);
+        return obj;
     }
 
     return createLogger(logger);
@@ -609,13 +690,13 @@ static PyObject*
 communicatorAddObjectFactory(CommunicatorObject* self, PyObject* args)
 {
     PyObject* factoryType = lookupType("Ice.ObjectFactory");
-    assert(factoryType != NULL);
+    assert(factoryType);
 
     PyObject* factory;
     char* id;
     if(!PyArg_ParseTuple(args, STRCAST("O!s"), factoryType, &factory, &id))
     {
-        return NULL;
+        return 0;
     }
 
     ObjectFactoryPtr pof;
@@ -627,13 +708,13 @@ communicatorAddObjectFactory(CommunicatorObject* self, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
 
     }
 
     if(!pof->add(factory, id))
     {
-        return NULL;
+        return 0;
     }
 
     Py_INCREF(Py_None);
@@ -649,7 +730,7 @@ communicatorFindObjectFactory(CommunicatorObject* self, PyObject* args)
     char* id;
     if(!PyArg_ParseTuple(args, STRCAST("s"), &id))
     {
-        return NULL;
+        return 0;
     }
 
     ObjectFactoryPtr pof;
@@ -661,8 +742,7 @@ communicatorFindObjectFactory(CommunicatorObject* self, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
-
+        return 0;
     }
 
     return pof->find(id);
@@ -674,16 +754,18 @@ extern "C"
 static PyObject*
 communicatorSetDefaultContext(CommunicatorObject* self, PyObject* args)
 {
+    PyErr_Warn(PyExc_DeprecationWarning, STRCAST("setDefaultContext is deprecated, use per-proxy contexts or implicit contexts (if applicable) instead."));
+
     PyObject* dict;
     if(!PyArg_ParseTuple(args, STRCAST("O!"), &PyDict_Type, &dict))
     {
-        return NULL;
+        return 0;
     }
 
     Ice::Context ctx;
     if(!dictionaryToContext(dict, ctx))
     {
-	return NULL;
+        return 0;
     }
 
     try
@@ -693,7 +775,7 @@ communicatorSetDefaultContext(CommunicatorObject* self, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
 
     }
 
@@ -707,6 +789,8 @@ extern "C"
 static PyObject*
 communicatorGetDefaultContext(CommunicatorObject* self)
 {
+    PyErr_Warn(PyExc_DeprecationWarning, STRCAST("getDefaultContext is deprecated, use per-proxy contexts or implicit contexts (if applicable) instead."));
+
     Ice::Context ctx;
     try
     {
@@ -715,23 +799,40 @@ communicatorGetDefaultContext(CommunicatorObject* self)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     PyObjectHandle dict = PyDict_New();
-    if(dict.get() == NULL)
+    if(!dict.get())
     {
-	return NULL;
+        return 0;
     }
 
     if(!contextToDictionary(ctx, dict.get()))
     {
-	return NULL;
+        return 0;
     }
 
     return dict.release();
 }
 
+#ifdef WIN32
+extern "C"
+#endif
+static PyObject*
+communicatorGetImplicitContext(CommunicatorObject* self)
+{
+    Ice::ImplicitContextPtr implicitContext = (*self->communicator)->getImplicitContext();
+    
+    if(implicitContext == 0)
+    {
+        return 0;
+    }
+
+    return createImplicitContext(implicitContext);
+}
+
+
 #ifdef WIN32
 extern "C"
 #endif
@@ -741,7 +842,7 @@ communicatorCreateObjectAdapter(CommunicatorObject* self, PyObject* args)
     char* name;
     if(!PyArg_ParseTuple(args, STRCAST("s"), &name))
     {
-        return NULL;
+        return 0;
     }
 
     assert(self->communicator);
@@ -753,11 +854,11 @@ communicatorCreateObjectAdapter(CommunicatorObject* self, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     PyObject* obj = createObjectAdapter(adapter);
-    if(obj == NULL)
+    if(!obj)
     {
         try
         {
@@ -781,7 +882,7 @@ communicatorCreateObjectAdapterWithEndpoints(CommunicatorObject* self, PyObject*
     char* endpoints;
     if(!PyArg_ParseTuple(args, STRCAST("ss"), &name, &endpoints))
     {
-        return NULL;
+        return 0;
     }
 
     assert(self->communicator);
@@ -793,11 +894,11 @@ communicatorCreateObjectAdapterWithEndpoints(CommunicatorObject* self, PyObject*
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     PyObject* obj = createObjectAdapter(adapter);
-    if(obj == NULL)
+    if(!obj)
     {
         try
         {
@@ -821,20 +922,20 @@ communicatorCreateObjectAdapterWithRouter(CommunicatorObject* self, PyObject* ar
     PyObject* proxy;
     if(!PyArg_ParseTuple(args, STRCAST("sO"), &name, &proxy))
     {
-	return NULL;
+        return 0;
     }
 
     PyObject* routerProxyType = lookupType("Ice.RouterPrx");
-    assert(routerProxyType != NULL);
+    assert(routerProxyType);
     Ice::RouterPrx router;
     if(PyObject_IsInstance(proxy, routerProxyType))
     {
-	router = Ice::RouterPrx::uncheckedCast(getProxy(proxy));
+        router = Ice::RouterPrx::uncheckedCast(getProxy(proxy));
     }
     else if(proxy != Py_None)
     {
-	PyErr_Format(PyExc_ValueError, STRCAST("ice_createObjectAdapterWithRouter requires None or Ice.RouterPrx"));
-	return NULL;
+        PyErr_Format(PyExc_ValueError, STRCAST("ice_createObjectAdapterWithRouter requires None or Ice.RouterPrx"));
+        return 0;
     }
 
     AllowThreads allowThreads; // Release Python's global interpreter lock to avoid a potential deadlock.
@@ -848,11 +949,11 @@ communicatorCreateObjectAdapterWithRouter(CommunicatorObject* self, PyObject* ar
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     PyObject* obj = createObjectAdapter(adapter);
-    if(obj == NULL)
+    if(!obj)
     {
         try
         {
@@ -881,17 +982,17 @@ communicatorGetDefaultRouter(CommunicatorObject* self)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     if(!router)
     {
-	Py_INCREF(Py_None);
-	return Py_None;
+        Py_INCREF(Py_None);
+        return Py_None;
     }
 
     PyObject* routerProxyType = lookupType("Ice.RouterPrx");
-    assert(routerProxyType != NULL);
+    assert(routerProxyType);
     return createProxy(router, *self->communicator, routerProxyType);
 }
 
@@ -904,31 +1005,31 @@ communicatorSetDefaultRouter(CommunicatorObject* self, PyObject* args)
     PyObject* proxy;
     if(!PyArg_ParseTuple(args, STRCAST("O"), &proxy))
     {
-	return NULL;
+        return 0;
     }
 
     PyObject* routerProxyType = lookupType("Ice.RouterPrx");
-    assert(routerProxyType != NULL);
+    assert(routerProxyType);
     Ice::RouterPrx router;
     if(PyObject_IsInstance(proxy, routerProxyType))
     {
-	router = Ice::RouterPrx::uncheckedCast(getProxy(proxy));
+        router = Ice::RouterPrx::uncheckedCast(getProxy(proxy));
     }
     else if(proxy != Py_None)
     {
-	PyErr_Format(PyExc_ValueError, STRCAST("ice_setDefaultRouter requires None or Ice.RouterPrx"));
-	return NULL;
+        PyErr_Format(PyExc_ValueError, STRCAST("ice_setDefaultRouter requires None or Ice.RouterPrx"));
+        return 0;
     }
 
     assert(self->communicator);
     try
     {
-	(*self->communicator)->setDefaultRouter(router);
+        (*self->communicator)->setDefaultRouter(router);
     }
     catch(const Ice::Exception& ex)
     {
-	setPythonException(ex);
-	return NULL;
+        setPythonException(ex);
+        return 0;
     }
 
     Py_INCREF(Py_None);
@@ -950,17 +1051,17 @@ communicatorGetDefaultLocator(CommunicatorObject* self)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     if(!locator)
     {
-	Py_INCREF(Py_None);
-	return Py_None;
+        Py_INCREF(Py_None);
+        return Py_None;
     }
 
     PyObject* locatorProxyType = lookupType("Ice.LocatorPrx");
-    assert(locatorProxyType != NULL);
+    assert(locatorProxyType);
     return createProxy(locator, *self->communicator, locatorProxyType);
 }
 
@@ -973,31 +1074,31 @@ communicatorSetDefaultLocator(CommunicatorObject* self, PyObject* args)
     PyObject* proxy;
     if(!PyArg_ParseTuple(args, STRCAST("O"), &proxy))
     {
-	return NULL;
+        return 0;
     }
 
     PyObject* locatorProxyType = lookupType("Ice.LocatorPrx");
-    assert(locatorProxyType != NULL);
+    assert(locatorProxyType);
     Ice::LocatorPrx locator;
     if(PyObject_IsInstance(proxy, locatorProxyType))
     {
-	locator = Ice::LocatorPrx::uncheckedCast(getProxy(proxy));
+        locator = Ice::LocatorPrx::uncheckedCast(getProxy(proxy));
     }
     else if(proxy != Py_None)
     {
-	PyErr_Format(PyExc_ValueError, STRCAST("ice_setDefaultLocator requires None or Ice.LocatorPrx"));
-	return NULL;
+        PyErr_Format(PyExc_ValueError, STRCAST("ice_setDefaultLocator requires None or Ice.LocatorPrx"));
+        return 0;
     }
 
     assert(self->communicator);
     try
     {
-	(*self->communicator)->setDefaultLocator(locator);
+        (*self->communicator)->setDefaultLocator(locator);
     }
     catch(const Ice::Exception& ex)
     {
-	setPythonException(ex);
-	return NULL;
+        setPythonException(ex);
+        return 0;
     }
 
     Py_INCREF(Py_None);
@@ -1006,55 +1107,61 @@ communicatorSetDefaultLocator(CommunicatorObject* self, PyObject* args)
 
 static PyMethodDef CommunicatorMethods[] =
 {
-    { STRCAST("destroy"), (PyCFunction)communicatorDestroy, METH_NOARGS,
+    { STRCAST("destroy"), reinterpret_cast<PyCFunction>(communicatorDestroy), METH_NOARGS,
         PyDoc_STR(STRCAST("destroy() -> None")) },
-    { STRCAST("shutdown"), (PyCFunction)communicatorShutdown, METH_NOARGS,
+    { STRCAST("shutdown"), reinterpret_cast<PyCFunction>(communicatorShutdown), METH_NOARGS,
         PyDoc_STR(STRCAST("shutdown() -> None")) },
-    { STRCAST("waitForShutdown"), (PyCFunction)communicatorWaitForShutdown, METH_VARARGS,
+    { STRCAST("waitForShutdown"), reinterpret_cast<PyCFunction>(communicatorWaitForShutdown), METH_VARARGS,
         PyDoc_STR(STRCAST("waitForShutdown() -> None")) },
-    { STRCAST("stringToProxy"), (PyCFunction)communicatorStringToProxy, METH_VARARGS,
+    { STRCAST("isShutdown"), reinterpret_cast<PyCFunction>(communicatorIsShutdown), METH_NOARGS,
+        PyDoc_STR(STRCAST("isShutdown() -> bool")) },
+    { STRCAST("stringToProxy"), reinterpret_cast<PyCFunction>(communicatorStringToProxy), METH_VARARGS,
         PyDoc_STR(STRCAST("stringToProxy(str) -> Ice.ObjectPrx")) },
-    { STRCAST("proxyToString"), (PyCFunction)communicatorProxyToString, METH_VARARGS,
+    { STRCAST("proxyToString"), reinterpret_cast<PyCFunction>(communicatorProxyToString), METH_VARARGS,
         PyDoc_STR(STRCAST("proxyToString(Ice.ObjectPrx) -> string")) },
-    { STRCAST("stringToIdentity"), (PyCFunction)communicatorStringToIdentity, METH_VARARGS,
+    { STRCAST("propertyToProxy"), reinterpret_cast<PyCFunction>(communicatorPropertyToProxy), METH_VARARGS,
+        PyDoc_STR(STRCAST("propertyToProxy(str) -> Ice.ObjectPrx")) },
+    { STRCAST("stringToIdentity"), reinterpret_cast<PyCFunction>(communicatorStringToIdentity), METH_VARARGS,
         PyDoc_STR(STRCAST("stringToIdentity(str) -> Ice.Identity")) },
-    { STRCAST("identityToString"), (PyCFunction)communicatorIdentityToString, METH_VARARGS,
+    { STRCAST("identityToString"), reinterpret_cast<PyCFunction>(communicatorIdentityToString), METH_VARARGS,
         PyDoc_STR(STRCAST("identityToString(Ice.Identity) -> string")) },
-    { STRCAST("createObjectAdapter"), (PyCFunction)communicatorCreateObjectAdapter, METH_VARARGS,
+    { STRCAST("createObjectAdapter"), reinterpret_cast<PyCFunction>(communicatorCreateObjectAdapter), METH_VARARGS,
         PyDoc_STR(STRCAST("createObjectAdapter(name) -> Ice.ObjectAdapter")) },
-    { STRCAST("createObjectAdapterWithEndpoints"), (PyCFunction)communicatorCreateObjectAdapterWithEndpoints,
-	METH_VARARGS,
-	PyDoc_STR(STRCAST("createObjectAdapterWithEndpoints(name, endpoints) -> Ice.ObjectAdapter")) },
-    { STRCAST("createObjectAdapterWithRouter"), (PyCFunction)communicatorCreateObjectAdapterWithRouter,
-	METH_VARARGS,
-	PyDoc_STR(STRCAST("createObjectAdapterWithRouter(name, router) -> Ice.ObjectAdapter")) },
-    { STRCAST("addObjectFactory"), (PyCFunction)communicatorAddObjectFactory, METH_VARARGS,
+    { STRCAST("createObjectAdapterWithEndpoints"),
+        reinterpret_cast<PyCFunction>(communicatorCreateObjectAdapterWithEndpoints), METH_VARARGS,
+        PyDoc_STR(STRCAST("createObjectAdapterWithEndpoints(name, endpoints) -> Ice.ObjectAdapter")) },
+    { STRCAST("createObjectAdapterWithRouter"),
+        reinterpret_cast<PyCFunction>(communicatorCreateObjectAdapterWithRouter), METH_VARARGS,
+        PyDoc_STR(STRCAST("createObjectAdapterWithRouter(name, router) -> Ice.ObjectAdapter")) },
+    { STRCAST("addObjectFactory"), reinterpret_cast<PyCFunction>(communicatorAddObjectFactory), METH_VARARGS,
         PyDoc_STR(STRCAST("addObjectFactory(factory, id) -> None")) },
-    { STRCAST("findObjectFactory"), (PyCFunction)communicatorFindObjectFactory, METH_VARARGS,
+    { STRCAST("findObjectFactory"), reinterpret_cast<PyCFunction>(communicatorFindObjectFactory), METH_VARARGS,
         PyDoc_STR(STRCAST("findObjectFactory(id) -> Ice.ObjectFactory")) },
-    { STRCAST("setDefaultContext"), (PyCFunction)communicatorSetDefaultContext, METH_VARARGS,
+    { STRCAST("setDefaultContext"), reinterpret_cast<PyCFunction>(communicatorSetDefaultContext), METH_VARARGS,
         PyDoc_STR(STRCAST("setDefaultContext(ctx) -> None")) },
-    { STRCAST("getDefaultContext"), (PyCFunction)communicatorGetDefaultContext, METH_NOARGS,
+    { STRCAST("getDefaultContext"), reinterpret_cast<PyCFunction>(communicatorGetDefaultContext), METH_NOARGS,
         PyDoc_STR(STRCAST("getDefaultContext() -> Ice.Context")) },
-    { STRCAST("getProperties"), (PyCFunction)communicatorGetProperties, METH_NOARGS,
+    { STRCAST("getImplicitContext"), reinterpret_cast<PyCFunction>(communicatorGetImplicitContext), METH_NOARGS,
+      PyDoc_STR(STRCAST("getImplicitContext() -> Ice.ImplicitContext")) },
+    { STRCAST("getProperties"), reinterpret_cast<PyCFunction>(communicatorGetProperties), METH_NOARGS,
         PyDoc_STR(STRCAST("getProperties() -> Ice.Properties")) },
-    { STRCAST("getLogger"), (PyCFunction)communicatorGetLogger, METH_NOARGS,
+    { STRCAST("getLogger"), reinterpret_cast<PyCFunction>(communicatorGetLogger), METH_NOARGS,
         PyDoc_STR(STRCAST("getLogger() -> Ice.Logger")) },
-    { STRCAST("getDefaultRouter"), (PyCFunction)communicatorGetDefaultRouter, METH_NOARGS,
+    { STRCAST("getDefaultRouter"), reinterpret_cast<PyCFunction>(communicatorGetDefaultRouter), METH_NOARGS,
         PyDoc_STR(STRCAST("getDefaultRouter() -> proxy")) },
-    { STRCAST("setDefaultRouter"), (PyCFunction)communicatorSetDefaultRouter, METH_VARARGS,
+    { STRCAST("setDefaultRouter"), reinterpret_cast<PyCFunction>(communicatorSetDefaultRouter), METH_VARARGS,
         PyDoc_STR(STRCAST("setDefaultRouter(proxy) -> None")) },
-    { STRCAST("getDefaultLocator"), (PyCFunction)communicatorGetDefaultLocator, METH_NOARGS,
+    { STRCAST("getDefaultLocator"), reinterpret_cast<PyCFunction>(communicatorGetDefaultLocator), METH_NOARGS,
         PyDoc_STR(STRCAST("getDefaultLocator() -> proxy")) },
-    { STRCAST("setDefaultLocator"), (PyCFunction)communicatorSetDefaultLocator, METH_VARARGS,
+    { STRCAST("setDefaultLocator"), reinterpret_cast<PyCFunction>(communicatorSetDefaultLocator), METH_VARARGS,
         PyDoc_STR(STRCAST("setDefaultLocator(proxy) -> None")) },
-    { STRCAST("flushBatchRequests"), (PyCFunction)communicatorFlushBatchRequests, METH_NOARGS,
+    { STRCAST("flushBatchRequests"), reinterpret_cast<PyCFunction>(communicatorFlushBatchRequests), METH_NOARGS,
         PyDoc_STR(STRCAST("flushBatchRequests() -> None")) },
-    { STRCAST("_setWrapper"), (PyCFunction)communicatorSetWrapper, METH_VARARGS,
+    { STRCAST("_setWrapper"), reinterpret_cast<PyCFunction>(communicatorSetWrapper), METH_VARARGS,
         PyDoc_STR(STRCAST("internal function")) },
-    { STRCAST("_getWrapper"), (PyCFunction)communicatorGetWrapper, METH_NOARGS,
+    { STRCAST("_getWrapper"), reinterpret_cast<PyCFunction>(communicatorGetWrapper), METH_NOARGS,
         PyDoc_STR(STRCAST("internal function")) },
-    { NULL, NULL} /* sentinel */
+    { 0, 0 } /* sentinel */
 };
 
 namespace IcePy
@@ -1064,13 +1171,13 @@ PyTypeObject CommunicatorType =
 {
     /* The ob_type field must be initialized in the module init function
      * to be portable to Windows without using C++. */
-    PyObject_HEAD_INIT(NULL)
+    PyObject_HEAD_INIT(0)
     0,                               /* ob_size */
     STRCAST("IcePy.Communicator"),   /* tp_name */
     sizeof(CommunicatorObject),      /* tp_basicsize */
     0,                               /* tp_itemsize */
     /* methods */
-    (destructor)communicatorDealloc, /* tp_dealloc */
+    reinterpret_cast<destructor>(communicatorDealloc), /* tp_dealloc */
     0,                               /* tp_print */
     0,                               /* tp_getattr */
     0,                               /* tp_setattr */
@@ -1101,9 +1208,9 @@ PyTypeObject CommunicatorType =
     0,                               /* tp_descr_get */
     0,                               /* tp_descr_set */
     0,                               /* tp_dictoffset */
-    (initproc)communicatorInit,      /* tp_init */
+    reinterpret_cast<initproc>(communicatorInit), /* tp_init */
     0,                               /* tp_alloc */
-    (newfunc)communicatorNew,        /* tp_new */
+    reinterpret_cast<newfunc>(communicatorNew), /* tp_new */
     0,                               /* tp_free */
     0,                               /* tp_is_gc */
 };
@@ -1119,7 +1226,8 @@ IcePy::initCommunicator(PyObject* module)
     {
         return false;
     }
-    if(PyModule_AddObject(module, STRCAST("Communicator"), (PyObject*)&CommunicatorType) < 0)
+    PyTypeObject* type = &CommunicatorType; // Necessary to prevent GCC's strict-alias warnings.
+    if(PyModule_AddObject(module, STRCAST("Communicator"), reinterpret_cast<PyObject*>(type)) < 0)
     {
         return false;
     }
@@ -1130,8 +1238,8 @@ IcePy::initCommunicator(PyObject* module)
 Ice::CommunicatorPtr
 IcePy::getCommunicator(PyObject* obj)
 {
-    assert(PyObject_IsInstance(obj, (PyObject*)&CommunicatorType));
-    CommunicatorObject* cobj = (CommunicatorObject*)obj;
+    assert(PyObject_IsInstance(obj, reinterpret_cast<PyObject*>(&CommunicatorType)));
+    CommunicatorObject* cobj = reinterpret_cast<CommunicatorObject*>(obj);
     return *cobj->communicator;
 }
 
@@ -1141,12 +1249,12 @@ IcePy::createCommunicator(const Ice::CommunicatorPtr& communicator)
     CommunicatorMap::iterator p = _communicatorMap.find(communicator);
     if(p != _communicatorMap.end())
     {
-	Py_INCREF(p->second);
-	return p->second;
+        Py_INCREF(p->second);
+        return p->second;
     }
 
-    CommunicatorObject* obj = communicatorNew(NULL);
-    if(obj != NULL)
+    CommunicatorObject* obj = communicatorNew(0);
+    if(obj)
     {
         obj->communicator = new Ice::CommunicatorPtr(communicator);
     }
@@ -1158,7 +1266,7 @@ IcePy::getCommunicatorWrapper(const Ice::CommunicatorPtr& communicator)
 {
     CommunicatorMap::iterator p = _communicatorMap.find(communicator);
     assert(p != _communicatorMap.end());
-    CommunicatorObject* obj = (CommunicatorObject*)p->second;
+    CommunicatorObject* obj = reinterpret_cast<CommunicatorObject*>(p->second);
     Py_INCREF(obj->wrapper);
     return obj->wrapper;
 }
