@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -48,4 +48,7 @@ PyObject* createLogger(const Ice::LoggerPtr&);
 
 }
 
+extern "C" PyObject* IcePy_getProcessLogger(PyObject*);
+extern "C" PyObject* IcePy_setProcessLogger(PyObject*, PyObject*);
+
 #endif
