@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -123,6 +123,8 @@ IcePy::ServantWrapper::ServantWrapper(PyObject* servant) :
 
 IcePy::ServantWrapper::~ServantWrapper()
 {
+    AdoptThread adoptThread; // Ensure the current thread is able to call into Python.
+
     Py_DECREF(_servant);
 }
 
@@ -157,7 +159,7 @@ IcePy::ServantWrapper::ice_invoke_async(const Ice::AMD_Object_ice_invokePtr& cb,
                 string attrName = "_op_" + current.operation;
                 PyObjectHandle h = PyObject_GetAttrString((PyObject*)_servant->ob_type,
                                                           const_cast<char*>(attrName.c_str()));
-                if(h.get() == NULL)
+                if(!h.get())
                 {
                     Ice::OperationNotExistException ex(__FILE__, __LINE__);
                     ex.id = current.id;
@@ -175,6 +177,8 @@ IcePy::ServantWrapper::ice_invoke_async(const Ice::AMD_Object_ice_invokePtr& cb,
             }
         }
 
+        __checkMode(op->mode(), current.mode);
+
         op->dispatch(_servant, cb, inParams, current);
     }
     catch(const Ice::Exception& ex)
@@ -209,7 +213,7 @@ IcePy::ServantLocatorWrapper::locate(const Ice::Current& current, Ice::LocalObje
 {
     CookiePtr c = new Cookie; // The Cookie constructor adopts this thread.
     c->current = createCurrent(current);
-    if(c->current == NULL)
+    if(!c->current)
     {
         throwPythonException();
     }
@@ -230,7 +234,7 @@ IcePy::ServantLocatorWrapper::locate(const Ice::Current& current, Ice::LocalObje
         return 0;
     }
 
-    PyObject* servantObj = NULL;
+    PyObject* servantObj = 0;
     PyObject* cookieObj = Py_None;
     if(PyTuple_Check(res.get()))
     {
@@ -279,7 +283,7 @@ IcePy::ServantLocatorWrapper::finished(const Ice::Current&, const Ice::ObjectPtr
     PyObjectHandle servantObj = wrapper->getObject();
 
     PyObjectHandle res = PyObject_CallMethod(_locator, STRCAST("finished"), STRCAST("OOO"), c->current,
-					     servantObj.get(), c->cookie);
+                                             servantObj.get(), c->cookie);
     if(PyErr_Occurred())
     {
         throwPythonException();
@@ -309,8 +313,8 @@ IcePy::ServantLocatorWrapper::getObject()
 
 IcePy::ServantLocatorWrapper::Cookie::Cookie()
 {
-    current = NULL;
-    cookie = NULL;
+    current = 0;
+    cookie = 0;
 }
 
 IcePy::ServantLocatorWrapper::Cookie::~Cookie()
@@ -326,7 +330,7 @@ static ObjectAdapterObject*
 adapterNew(PyObject* /*arg*/)
 {
     PyErr_Format(PyExc_RuntimeError, STRCAST("Use communicator.createObjectAdapter to create an adapter"));
-    return NULL;
+    return 0;
 }
 
 #ifdef WIN32
@@ -366,7 +370,7 @@ adapterGetName(ObjectAdapterObject* self)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     return PyString_FromString(const_cast<char*>(name.c_str()));
@@ -387,7 +391,7 @@ adapterGetCommunicator(ObjectAdapterObject* self)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     return createCommunicator(communicator);
@@ -417,7 +421,7 @@ adapterActivate(ObjectAdapterObject* self)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     Py_INCREF(Py_None);
@@ -438,7 +442,7 @@ adapterHold(ObjectAdapterObject* self)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     Py_INCREF(Py_None);
@@ -463,7 +467,7 @@ adapterWaitForHold(ObjectAdapterObject* self, PyObject* args)
     int timeout = 0;
     if(!PyArg_ParseTuple(args, STRCAST("i"), &timeout))
     {
-        return NULL;
+        return 0;
     }
 
     assert(timeout > 0);
@@ -496,8 +500,7 @@ adapterWaitForHold(ObjectAdapterObject* self, PyObject* args)
 
             if(!done)
             {
-                Py_INCREF(Py_False);
-                return Py_False;
+                PyRETURN_FALSE;
             }
         }
 
@@ -507,7 +510,7 @@ adapterWaitForHold(ObjectAdapterObject* self, PyObject* args)
         if(ex)
         {
             setPythonException(*ex);
-            return NULL;
+            return 0;
         }
     }
     else
@@ -520,12 +523,11 @@ adapterWaitForHold(ObjectAdapterObject* self, PyObject* args)
         catch(const Ice::Exception& ex)
         {
             setPythonException(ex);
-            return NULL;
+            return 0;
         }
     }
 
-    Py_INCREF(Py_True);
-    return Py_True;
+    PyRETURN_TRUE;
 }
 
 #ifdef WIN32
@@ -543,7 +545,7 @@ adapterDeactivate(ObjectAdapterObject* self)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     Py_INCREF(Py_None);
@@ -568,7 +570,7 @@ adapterWaitForDeactivate(ObjectAdapterObject* self, PyObject* args)
     int timeout = 0;
     if(!PyArg_ParseTuple(args, STRCAST("i"), &timeout))
     {
-        return NULL;
+        return 0;
     }
 
     assert(timeout > 0);
@@ -592,17 +594,19 @@ adapterWaitForDeactivate(ObjectAdapterObject* self, PyObject* args)
                 self->deactivateThread = new AdapterInvokeThreadPtr(t);
                 t->start();
             }
-
-            bool done;
+            
+            while(!self->deactivated)
             {
-                AllowThreads allowThreads; // Release Python's global interpreter lock during blocking calls.
-                done = (*self->deactivateMonitor).timedWait(IceUtil::Time::milliSeconds(timeout));
-            }
-
-            if(!done)
-            {
-                Py_INCREF(Py_False);
-                return Py_False;
+                bool done;
+                {
+                    AllowThreads allowThreads; // Release Python's global interpreter lock during blocking calls.
+                    done = (*self->deactivateMonitor).timedWait(IceUtil::Time::milliSeconds(timeout));
+                }
+                
+                if(!done)
+                {
+                    PyRETURN_FALSE;
+                }
             }
         }
 
@@ -612,7 +616,7 @@ adapterWaitForDeactivate(ObjectAdapterObject* self, PyObject* args)
         if(ex)
         {
             setPythonException(*ex);
-            return NULL;
+            return 0;
         }
     }
     else
@@ -625,12 +629,54 @@ adapterWaitForDeactivate(ObjectAdapterObject* self, PyObject* args)
         catch(const Ice::Exception& ex)
         {
             setPythonException(ex);
-            return NULL;
+            return 0;
         }
     }
 
-    Py_INCREF(Py_True);
-    return Py_True;
+    PyRETURN_TRUE;
+}
+
+#ifdef WIN32
+extern "C"
+#endif
+static PyObject*
+adapterIsDeactivated(ObjectAdapterObject* self)
+{
+    assert(self->adapter);
+    try
+    {
+        (*self->adapter)->isDeactivated();
+    }
+    catch(const Ice::Exception& ex)
+    {
+        setPythonException(ex);
+        return 0;
+    }
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+#ifdef WIN32
+extern "C"
+#endif
+static PyObject*
+adapterDestroy(ObjectAdapterObject* self)
+{
+    assert(self->adapter);
+    try
+    {
+        AllowThreads allowThreads; // Release Python's global interpreter lock during blocking calls.
+        (*self->adapter)->destroy();
+    }
+    catch(const Ice::Exception& ex)
+    {
+        setPythonException(ex);
+        return 0;
+    }
+
+    Py_INCREF(Py_None);
+    return Py_None;
 }
 
 #ifdef WIN32
@@ -645,19 +691,19 @@ adapterAdd(ObjectAdapterObject* self, PyObject* args)
     PyObject* id;
     if(!PyArg_ParseTuple(args, STRCAST("O!O!"), objectType, &servant, identityType, &id))
     {
-        return NULL;
+        return 0;
     }
 
     Ice::Identity ident;
     if(!getIdentity(id, ident))
     {
-        return NULL;
+        return 0;
     }
 
     ServantWrapperPtr wrapper = new ServantWrapper(servant);
     if(PyErr_Occurred())
     {
-        return NULL;
+        return 0;
     }
 
     assert(self->adapter);
@@ -669,7 +715,7 @@ adapterAdd(ObjectAdapterObject* self, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     return createProxy(proxy, (*self->adapter)->getCommunicator());
@@ -688,19 +734,19 @@ adapterAddFacet(ObjectAdapterObject* self, PyObject* args)
     char* facet;
     if(!PyArg_ParseTuple(args, STRCAST("O!O!s"), objectType, &servant, identityType, &id, &facet))
     {
-        return NULL;
+        return 0;
     }
 
     Ice::Identity ident;
     if(!getIdentity(id, ident))
     {
-        return NULL;
+        return 0;
     }
 
     ServantWrapperPtr wrapper = new ServantWrapper(servant);
     if(PyErr_Occurred())
     {
-        return NULL;
+        return 0;
     }
 
     assert(self->adapter);
@@ -712,7 +758,7 @@ adapterAddFacet(ObjectAdapterObject* self, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     return createProxy(proxy, (*self->adapter)->getCommunicator());
@@ -728,13 +774,13 @@ adapterAddWithUUID(ObjectAdapterObject* self, PyObject* args)
     PyObject* servant;
     if(!PyArg_ParseTuple(args, STRCAST("O!"), objectType, &servant))
     {
-        return NULL;
+        return 0;
     }
 
     ServantWrapperPtr wrapper = new ServantWrapper(servant);
     if(PyErr_Occurred())
     {
-        return NULL;
+        return 0;
     }
 
     assert(self->adapter);
@@ -746,7 +792,7 @@ adapterAddWithUUID(ObjectAdapterObject* self, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     return createProxy(proxy, (*self->adapter)->getCommunicator());
@@ -763,13 +809,13 @@ adapterAddFacetWithUUID(ObjectAdapterObject* self, PyObject* args)
     char* facet;
     if(!PyArg_ParseTuple(args, STRCAST("O!s"), objectType, &servant, &facet))
     {
-        return NULL;
+        return 0;
     }
 
     ServantWrapperPtr wrapper = new ServantWrapper(servant);
     if(PyErr_Occurred())
     {
-        return NULL;
+        return 0;
     }
 
     assert(self->adapter);
@@ -781,7 +827,7 @@ adapterAddFacetWithUUID(ObjectAdapterObject* self, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     return createProxy(proxy, (*self->adapter)->getCommunicator());
@@ -797,13 +843,13 @@ adapterRemove(ObjectAdapterObject* self, PyObject* args)
     PyObject* id;
     if(!PyArg_ParseTuple(args, STRCAST("O!"), identityType, &id))
     {
-        return NULL;
+        return 0;
     }
 
     Ice::Identity ident;
     if(!getIdentity(id, ident))
     {
-        return NULL;
+        return 0;
     }
 
     assert(self->adapter);
@@ -815,13 +861,13 @@ adapterRemove(ObjectAdapterObject* self, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     if(!obj)
     {
-	Py_INCREF(Py_None);
-	return Py_None;
+        Py_INCREF(Py_None);
+        return Py_None;
     }
 
     ServantWrapperPtr wrapper = ServantWrapperPtr::dynamicCast(obj);
@@ -840,13 +886,13 @@ adapterRemoveFacet(ObjectAdapterObject* self, PyObject* args)
     char* facet;
     if(!PyArg_ParseTuple(args, STRCAST("O!s"), identityType, &id, &facet))
     {
-        return NULL;
+        return 0;
     }
 
     Ice::Identity ident;
     if(!getIdentity(id, ident))
     {
-        return NULL;
+        return 0;
     }
 
     assert(self->adapter);
@@ -858,13 +904,13 @@ adapterRemoveFacet(ObjectAdapterObject* self, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     if(!obj)
     {
-	Py_INCREF(Py_None);
-	return Py_None;
+        Py_INCREF(Py_None);
+        return Py_None;
     }
 
     ServantWrapperPtr wrapper = ServantWrapperPtr::dynamicCast(obj);
@@ -882,13 +928,13 @@ adapterRemoveAllFacets(ObjectAdapterObject* self, PyObject* args)
     PyObject* id;
     if(!PyArg_ParseTuple(args, STRCAST("O!"), identityType, &id))
     {
-        return NULL;
+        return 0;
     }
 
     Ice::Identity ident;
     if(!getIdentity(id, ident))
     {
-        return NULL;
+        return 0;
     }
 
     assert(self->adapter);
@@ -900,13 +946,13 @@ adapterRemoveAllFacets(ObjectAdapterObject* self, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     PyObjectHandle result = PyDict_New();
-    if(result.get() == NULL)
+    if(!result.get())
     {
-        return NULL;
+        return 0;
     }
 
     for(Ice::FacetMap::iterator p = facetMap.begin(); p != facetMap.end(); ++p)
@@ -916,7 +962,7 @@ adapterRemoveAllFacets(ObjectAdapterObject* self, PyObject* args)
         PyObjectHandle obj = wrapper->getObject();
         if(PyDict_SetItemString(result.get(), const_cast<char*>(p->first.c_str()), obj.get()) < 0)
         {
-            return NULL;
+            return 0;
         }
     }
 
@@ -933,13 +979,13 @@ adapterFind(ObjectAdapterObject* self, PyObject* args)
     PyObject* id;
     if(!PyArg_ParseTuple(args, STRCAST("O!"), identityType, &id))
     {
-        return NULL;
+        return 0;
     }
 
     Ice::Identity ident;
     if(!getIdentity(id, ident))
     {
-        return NULL;
+        return 0;
     }
 
     assert(self->adapter);
@@ -951,13 +997,13 @@ adapterFind(ObjectAdapterObject* self, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     if(!obj)
     {
-	Py_INCREF(Py_None);
-	return Py_None;
+        Py_INCREF(Py_None);
+        return Py_None;
     }
 
     ServantWrapperPtr wrapper = ServantWrapperPtr::dynamicCast(obj);
@@ -976,13 +1022,13 @@ adapterFindFacet(ObjectAdapterObject* self, PyObject* args)
     char* facet;
     if(!PyArg_ParseTuple(args, STRCAST("O!s"), identityType, &id, &facet))
     {
-        return NULL;
+        return 0;
     }
 
     Ice::Identity ident;
     if(!getIdentity(id, ident))
     {
-        return NULL;
+        return 0;
     }
 
     assert(self->adapter);
@@ -994,13 +1040,13 @@ adapterFindFacet(ObjectAdapterObject* self, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     if(!obj)
     {
-	Py_INCREF(Py_None);
-	return Py_None;
+        Py_INCREF(Py_None);
+        return Py_None;
     }
 
     ServantWrapperPtr wrapper = ServantWrapperPtr::dynamicCast(obj);
@@ -1018,13 +1064,13 @@ adapterFindAllFacets(ObjectAdapterObject* self, PyObject* args)
     PyObject* id;
     if(!PyArg_ParseTuple(args, STRCAST("O!"), identityType, &id))
     {
-        return NULL;
+        return 0;
     }
 
     Ice::Identity ident;
     if(!getIdentity(id, ident))
     {
-        return NULL;
+        return 0;
     }
 
     assert(self->adapter);
@@ -1036,13 +1082,13 @@ adapterFindAllFacets(ObjectAdapterObject* self, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     PyObjectHandle result = PyDict_New();
-    if(result.get() == NULL)
+    if(!result.get())
     {
-        return NULL;
+        return 0;
     }
 
     for(Ice::FacetMap::iterator p = facetMap.begin(); p != facetMap.end(); ++p)
@@ -1052,7 +1098,7 @@ adapterFindAllFacets(ObjectAdapterObject* self, PyObject* args)
         PyObjectHandle obj = wrapper->getObject();
         if(PyDict_SetItemString(result.get(), const_cast<char*>(p->first.c_str()), obj.get()) < 0)
         {
-            return NULL;
+            return 0;
         }
     }
 
@@ -1069,7 +1115,7 @@ adapterFindByProxy(ObjectAdapterObject* self, PyObject* args)
     PyObject* proxy;
     if(!PyArg_ParseTuple(args, STRCAST("O!"), proxyType, &proxy))
     {
-        return NULL;
+        return 0;
     }
 
     Ice::ObjectPrx prx = getProxy(proxy);
@@ -1083,13 +1129,13 @@ adapterFindByProxy(ObjectAdapterObject* self, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     if(!obj)
     {
-	Py_INCREF(Py_None);
-	return Py_None;
+        Py_INCREF(Py_None);
+        return Py_None;
     }
 
     ServantWrapperPtr wrapper = ServantWrapperPtr::dynamicCast(obj);
@@ -1108,7 +1154,7 @@ adapterAddServantLocator(ObjectAdapterObject* self, PyObject* args)
     char* category;
     if(!PyArg_ParseTuple(args, STRCAST("O!s"), locatorType, &locator, &category))
     {
-        return NULL;
+        return 0;
     }
 
     ServantLocatorWrapperPtr wrapper = new ServantLocatorWrapper(locator);
@@ -1121,7 +1167,7 @@ adapterAddServantLocator(ObjectAdapterObject* self, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     Py_INCREF(Py_None);
@@ -1137,7 +1183,7 @@ adapterFindServantLocator(ObjectAdapterObject* self, PyObject* args)
     char* category;
     if(!PyArg_ParseTuple(args, STRCAST("s"), &category))
     {
-        return NULL;
+        return 0;
     }
 
     assert(self->adapter);
@@ -1149,13 +1195,13 @@ adapterFindServantLocator(ObjectAdapterObject* self, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     if(!locator)
     {
-	Py_INCREF(Py_None);
-	return Py_None;
+        Py_INCREF(Py_None);
+        return Py_None;
     }
 
     ServantLocatorWrapperPtr wrapper = ServantLocatorWrapperPtr::dynamicCast(locator);
@@ -1173,13 +1219,13 @@ adapterCreateProxy(ObjectAdapterObject* self, PyObject* args)
     PyObject* id;
     if(!PyArg_ParseTuple(args, STRCAST("O!"), identityType, &id))
     {
-        return NULL;
+        return 0;
     }
 
     Ice::Identity ident;
     if(!getIdentity(id, ident))
     {
-        return NULL;
+        return 0;
     }
 
     assert(self->adapter);
@@ -1191,7 +1237,7 @@ adapterCreateProxy(ObjectAdapterObject* self, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     return createProxy(proxy, (*self->adapter)->getCommunicator());
@@ -1207,13 +1253,13 @@ adapterCreateDirectProxy(ObjectAdapterObject* self, PyObject* args)
     PyObject* id;
     if(!PyArg_ParseTuple(args, STRCAST("O!"), identityType, &id))
     {
-        return NULL;
+        return 0;
     }
 
     Ice::Identity ident;
     if(!getIdentity(id, ident))
     {
-        return NULL;
+        return 0;
     }
 
     assert(self->adapter);
@@ -1225,7 +1271,41 @@ adapterCreateDirectProxy(ObjectAdapterObject* self, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
+    }
+
+    return createProxy(proxy, (*self->adapter)->getCommunicator());
+}
+
+#ifdef WIN32
+extern "C"
+#endif
+static PyObject*
+adapterCreateIndirectProxy(ObjectAdapterObject* self, PyObject* args)
+{
+    PyObject* identityType = lookupType("Ice.Identity");
+    PyObject* id;
+    if(!PyArg_ParseTuple(args, STRCAST("O!"), identityType, &id))
+    {
+        return 0;
+    }
+
+    Ice::Identity ident;
+    if(!getIdentity(id, ident))
+    {
+        return 0;
+    }
+
+    assert(self->adapter);
+    Ice::ObjectPrx proxy;
+    try
+    {
+        proxy = (*self->adapter)->createIndirectProxy(ident);
+    }
+    catch(const Ice::Exception& ex)
+    {
+        setPythonException(ex);
+        return 0;
     }
 
     return createProxy(proxy, (*self->adapter)->getCommunicator());
@@ -1241,13 +1321,13 @@ adapterCreateReverseProxy(ObjectAdapterObject* self, PyObject* args)
     PyObject* id;
     if(!PyArg_ParseTuple(args, STRCAST("O!"), identityType, &id))
     {
-        return NULL;
+        return 0;
     }
 
     Ice::Identity ident;
     if(!getIdentity(id, ident))
     {
-        return NULL;
+        return 0;
     }
 
     assert(self->adapter);
@@ -1259,7 +1339,7 @@ adapterCreateReverseProxy(ObjectAdapterObject* self, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     return createProxy(proxy, (*self->adapter)->getCommunicator());
@@ -1275,7 +1355,7 @@ adapterSetLocator(ObjectAdapterObject* self, PyObject* args)
     PyObject* proxy;
     if(!PyArg_ParseTuple(args, STRCAST("O!"), proxyType, &proxy))
     {
-        return NULL;
+        return 0;
     }
 
     Ice::LocatorPrx locator = Ice::LocatorPrx::uncheckedCast(getProxy(proxy));
@@ -1289,7 +1369,7 @@ adapterSetLocator(ObjectAdapterObject* self, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     Py_INCREF(Py_None);
@@ -1298,55 +1378,61 @@ adapterSetLocator(ObjectAdapterObject* self, PyObject* args)
 
 static PyMethodDef AdapterMethods[] =
 {
-    { STRCAST("getName"), (PyCFunction)adapterGetName, METH_NOARGS,
+    { STRCAST("getName"), reinterpret_cast<PyCFunction>(adapterGetName), METH_NOARGS,
         PyDoc_STR(STRCAST("getName() -> string")) },
-    { STRCAST("getCommunicator"), (PyCFunction)adapterGetCommunicator, METH_NOARGS,
+    { STRCAST("getCommunicator"), reinterpret_cast<PyCFunction>(adapterGetCommunicator), METH_NOARGS,
         PyDoc_STR(STRCAST("getCommunicator() -> Ice.Communicator")) },
-    { STRCAST("activate"), (PyCFunction)adapterActivate, METH_NOARGS,
+    { STRCAST("activate"), reinterpret_cast<PyCFunction>(adapterActivate), METH_NOARGS,
         PyDoc_STR(STRCAST("activate() -> None")) },
-    { STRCAST("hold"), (PyCFunction)adapterHold, METH_NOARGS,
+    { STRCAST("hold"), reinterpret_cast<PyCFunction>(adapterHold), METH_NOARGS,
         PyDoc_STR(STRCAST("hold() -> None")) },
-    { STRCAST("waitForHold"), (PyCFunction)adapterWaitForHold, METH_VARARGS,
+    { STRCAST("waitForHold"), reinterpret_cast<PyCFunction>(adapterWaitForHold), METH_VARARGS,
         PyDoc_STR(STRCAST("waitForHold() -> None")) },
-    { STRCAST("deactivate"), (PyCFunction)adapterDeactivate, METH_NOARGS,
+    { STRCAST("deactivate"), reinterpret_cast<PyCFunction>(adapterDeactivate), METH_NOARGS,
         PyDoc_STR(STRCAST("deactivate() -> None")) },
-    { STRCAST("waitForDeactivate"), (PyCFunction)adapterWaitForDeactivate, METH_VARARGS,
+    { STRCAST("waitForDeactivate"), reinterpret_cast<PyCFunction>(adapterWaitForDeactivate), METH_VARARGS,
         PyDoc_STR(STRCAST("waitForDeactivate() -> None")) },
-    { STRCAST("add"), (PyCFunction)adapterAdd, METH_VARARGS,
+    { STRCAST("isDeactivated"), reinterpret_cast<PyCFunction>(adapterIsDeactivated), METH_NOARGS,
+        PyDoc_STR(STRCAST("isDeactivatied() -> None")) },
+    { STRCAST("destroy"), reinterpret_cast<PyCFunction>(adapterDestroy), METH_NOARGS,
+        PyDoc_STR(STRCAST("destroy() -> None")) },
+    { STRCAST("add"), reinterpret_cast<PyCFunction>(adapterAdd), METH_VARARGS,
         PyDoc_STR(STRCAST("add(servant, identity) -> Ice.ObjectPrx")) },
-    { STRCAST("addFacet"), (PyCFunction)adapterAddFacet, METH_VARARGS,
+    { STRCAST("addFacet"), reinterpret_cast<PyCFunction>(adapterAddFacet), METH_VARARGS,
         PyDoc_STR(STRCAST("addFacet(servant, identity, facet) -> Ice.ObjectPrx")) },
-    { STRCAST("addWithUUID"), (PyCFunction)adapterAddWithUUID, METH_VARARGS,
+    { STRCAST("addWithUUID"), reinterpret_cast<PyCFunction>(adapterAddWithUUID), METH_VARARGS,
         PyDoc_STR(STRCAST("addWithUUID(servant) -> Ice.ObjectPrx")) },
-    { STRCAST("addFacetWithUUID"), (PyCFunction)adapterAddFacetWithUUID, METH_VARARGS,
+    { STRCAST("addFacetWithUUID"), reinterpret_cast<PyCFunction>(adapterAddFacetWithUUID), METH_VARARGS,
         PyDoc_STR(STRCAST("addFacetWithUUID(servant, facet) -> Ice.ObjectPrx")) },
-    { STRCAST("remove"), (PyCFunction)adapterRemove, METH_VARARGS,
+    { STRCAST("remove"), reinterpret_cast<PyCFunction>(adapterRemove), METH_VARARGS,
         PyDoc_STR(STRCAST("remove(identity) -> Ice.Object")) },
-    { STRCAST("removeFacet"), (PyCFunction)adapterRemoveFacet, METH_VARARGS,
+    { STRCAST("removeFacet"), reinterpret_cast<PyCFunction>(adapterRemoveFacet), METH_VARARGS,
         PyDoc_STR(STRCAST("removeFacet(identity, facet) -> Ice.Object")) },
-    { STRCAST("removeAllFacets"), (PyCFunction)adapterRemoveAllFacets, METH_VARARGS,
+    { STRCAST("removeAllFacets"), reinterpret_cast<PyCFunction>(adapterRemoveAllFacets), METH_VARARGS,
         PyDoc_STR(STRCAST("removeAllFacets(identity) -> dictionary")) },
-    { STRCAST("find"), (PyCFunction)adapterFind, METH_VARARGS,
+    { STRCAST("find"), reinterpret_cast<PyCFunction>(adapterFind), METH_VARARGS,
         PyDoc_STR(STRCAST("find(identity) -> Ice.Object")) },
-    { STRCAST("findFacet"), (PyCFunction)adapterFindFacet, METH_VARARGS,
+    { STRCAST("findFacet"), reinterpret_cast<PyCFunction>(adapterFindFacet), METH_VARARGS,
         PyDoc_STR(STRCAST("findFacet(identity, facet) -> Ice.Object")) },
-    { STRCAST("findAllFacets"), (PyCFunction)adapterFindAllFacets, METH_VARARGS,
+    { STRCAST("findAllFacets"), reinterpret_cast<PyCFunction>(adapterFindAllFacets), METH_VARARGS,
         PyDoc_STR(STRCAST("findAllFacets(identity) -> dictionary")) },
-    { STRCAST("findByProxy"), (PyCFunction)adapterFindByProxy, METH_VARARGS,
+    { STRCAST("findByProxy"), reinterpret_cast<PyCFunction>(adapterFindByProxy), METH_VARARGS,
         PyDoc_STR(STRCAST("findByProxy(Ice.ObjectPrx) -> Ice.Object")) },
-    { STRCAST("addServantLocator"), (PyCFunction)adapterAddServantLocator, METH_VARARGS,
+    { STRCAST("addServantLocator"), reinterpret_cast<PyCFunction>(adapterAddServantLocator), METH_VARARGS,
         PyDoc_STR(STRCAST("addServantLocator(Ice.ServantLocator, category) -> None")) },
-    { STRCAST("findServantLocator"), (PyCFunction)adapterFindServantLocator, METH_VARARGS,
+    { STRCAST("findServantLocator"), reinterpret_cast<PyCFunction>(adapterFindServantLocator), METH_VARARGS,
         PyDoc_STR(STRCAST("findServantLocator(category) -> Ice.ServantLocator")) },
-    { STRCAST("createProxy"), (PyCFunction)adapterCreateProxy, METH_VARARGS,
+    { STRCAST("createProxy"), reinterpret_cast<PyCFunction>(adapterCreateProxy), METH_VARARGS,
         PyDoc_STR(STRCAST("createProxy(identity) -> Ice.ObjectPrx")) },
-    { STRCAST("createDirectProxy"), (PyCFunction)adapterCreateDirectProxy, METH_VARARGS,
+    { STRCAST("createDirectProxy"), reinterpret_cast<PyCFunction>(adapterCreateDirectProxy), METH_VARARGS,
         PyDoc_STR(STRCAST("createDirectProxy(identity) -> Ice.ObjectPrx")) },
-    { STRCAST("createReverseProxy"), (PyCFunction)adapterCreateReverseProxy, METH_VARARGS,
+    { STRCAST("createIndirectProxy"), reinterpret_cast<PyCFunction>(adapterCreateIndirectProxy), METH_VARARGS,
+        PyDoc_STR(STRCAST("createIndirectProxy(identity) -> Ice.ObjectPrx")) },
+    { STRCAST("createReverseProxy"), reinterpret_cast<PyCFunction>(adapterCreateReverseProxy), METH_VARARGS,
         PyDoc_STR(STRCAST("createReverseProxy(identity) -> Ice.ObjectPrx")) },
-    { STRCAST("setLocator"), (PyCFunction)adapterSetLocator, METH_VARARGS,
+    { STRCAST("setLocator"), reinterpret_cast<PyCFunction>(adapterSetLocator), METH_VARARGS,
         PyDoc_STR(STRCAST("setLocator(proxy) -> None")) },
-    { NULL, NULL} /* sentinel */
+    { 0, 0 } /* sentinel */
 };
 
 namespace IcePy
@@ -1356,13 +1442,13 @@ PyTypeObject ObjectAdapterType =
 {
     /* The ob_type field must be initialized in the module init function
      * to be portable to Windows without using C++. */
-    PyObject_HEAD_INIT(NULL)
+    PyObject_HEAD_INIT(0)
     0,                               /* ob_size */
     STRCAST("IcePy.ObjectAdapter"),  /* tp_name */
     sizeof(ObjectAdapterObject),     /* tp_basicsize */
     0,                               /* tp_itemsize */
     /* methods */
-    (destructor)adapterDealloc,      /* tp_dealloc */
+    reinterpret_cast<destructor>(adapterDealloc), /* tp_dealloc */
     0,                               /* tp_print */
     0,                               /* tp_getattr */
     0,                               /* tp_setattr */
@@ -1395,7 +1481,7 @@ PyTypeObject ObjectAdapterType =
     0,                               /* tp_dictoffset */
     0,                               /* tp_init */
     0,                               /* tp_alloc */
-    (newfunc)adapterNew,             /* tp_new */
+    reinterpret_cast<newfunc>(adapterNew), /* tp_new */
     0,                               /* tp_free */
     0,                               /* tp_is_gc */
 };
@@ -1411,7 +1497,8 @@ IcePy::initObjectAdapter(PyObject* module)
     {
         return false;
     }
-    if(PyModule_AddObject(module, STRCAST("ObjectAdapter"), (PyObject*)&ObjectAdapterType) < 0)
+    PyTypeObject* type = &ObjectAdapterType; // Necessary to prevent GCC's strict-alias warnings.
+    if(PyModule_AddObject(module, STRCAST("ObjectAdapter"), reinterpret_cast<PyObject*>(type)) < 0)
     {
         return false;
     }
@@ -1423,7 +1510,7 @@ PyObject*
 IcePy::createObjectAdapter(const Ice::ObjectAdapterPtr& adapter)
 {
     ObjectAdapterObject* obj = PyObject_New(ObjectAdapterObject, &ObjectAdapterType);
-    if (obj != NULL)
+    if(obj)
     {
         obj->adapter = new Ice::ObjectAdapterPtr(adapter);
         obj->deactivateMonitor = new IceUtil::Monitor<IceUtil::Mutex>;
@@ -1433,14 +1520,14 @@ IcePy::createObjectAdapter(const Ice::ObjectAdapterPtr& adapter)
         obj->holdThread = 0;
         obj->held = false;
     }
-    return (PyObject*)obj;
+    return reinterpret_cast<PyObject*>(obj);
 }
 
 Ice::ObjectAdapterPtr
 IcePy::getObjectAdapter(PyObject* obj)
 {
-    assert(PyObject_IsInstance(obj, (PyObject*)&ObjectAdapterType));
-    ObjectAdapterObject* oaobj = (ObjectAdapterObject*)obj;
+    assert(PyObject_IsInstance(obj, reinterpret_cast<PyObject*>(&ObjectAdapterType)));
+    ObjectAdapterObject* oaobj = reinterpret_cast<ObjectAdapterObject*>(obj);
     return *oaobj->adapter;
 }
 
@@ -1451,28 +1538,30 @@ IcePy::wrapObjectAdapter(const Ice::ObjectAdapterPtr& adapter)
     // Create an Ice.ObjectAdapter wrapper for IcePy.ObjectAdapter.
     //
     PyObjectHandle adapterI = createObjectAdapter(adapter);
-    if(adapterI.get() == NULL)
+    if(!adapterI.get())
     {
-	return NULL;
+        return 0;
     }
     PyObject* wrapperType = lookupType("Ice.ObjectAdapterI");
-    assert(wrapperType != NULL);
+    assert(wrapperType);
     PyObjectHandle args = PyTuple_New(1);
-    if(args.get() == NULL)
+    if(!args.get())
     {
-	return NULL;
+        return 0;
     }
     PyTuple_SET_ITEM(args.get(), 0, adapterI.release());
-    return PyObject_Call(wrapperType, args.get(), NULL);
+    return PyObject_Call(wrapperType, args.get(), 0);
 }
 
 Ice::ObjectAdapterPtr
 IcePy::unwrapObjectAdapter(PyObject* obj)
 {
+#ifndef NDEBUG
     PyObject* wrapperType = lookupType("Ice.ObjectAdapterI");
-    assert(wrapperType != NULL);
+#endif
+    assert(wrapperType);
     assert(PyObject_IsInstance(obj, wrapperType));
     PyObjectHandle impl = PyObject_GetAttrString(obj, STRCAST("_impl"));
-    assert(impl.get() != NULL);
+    assert(impl.get());
     return getObjectAdapter(impl.get());
 }
