@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -38,6 +38,7 @@ public:
 
     virtual void unmarshaled(PyObject*, PyObject*, void*);
 
+    Ice::StringSeq metaData;
     TypeInfoPtr type;
 };
 typedef IceUtil::Handle<ParamInfo> ParamInfoPtr;
@@ -47,11 +48,12 @@ class OperationI : public Operation
 {
 public:
 
-    OperationI(const char*, PyObject*, int, PyObject*, PyObject*, PyObject*, PyObject*);
+    OperationI(const char*, PyObject*, PyObject*, int, PyObject*, PyObject*, PyObject*, PyObject*, PyObject*);
 
     virtual PyObject* invoke(const Ice::ObjectPrx&, PyObject*, PyObject*);
     virtual PyObject* invokeAsync(const Ice::ObjectPrx&, PyObject*, PyObject*, PyObject*);
     virtual void deprecate(const string&);
+    virtual Ice::OperationMode mode() const;
 
     virtual void dispatch(PyObject*, const Ice::AMD_Object_ice_invokePtr&, const vector<Ice::Byte>&,
                           const Ice::Current&);
@@ -66,7 +68,9 @@ private:
 
     string _name;
     Ice::OperationMode _mode;
+    Ice::OperationMode _sendMode;
     bool _amd;
+    Ice::StringSeq _metaData;
     ParamInfoList _inParams;
     ParamInfoList _outParams;
     ParamInfoPtr _returnType;
@@ -81,6 +85,7 @@ private:
     PyObject* unmarshalException(const vector<Ice::Byte>&, const Ice::CommunicatorPtr&);
     bool validateException(PyObject*) const;
     void checkTwowayOnly(const Ice::ObjectPrx&) const;
+    static void convertParams(PyObject*, ParamInfoList&, bool&);
 };
 typedef IceUtil::Handle<OperationI> OperationIPtr;
 
@@ -127,9 +132,9 @@ static OperationObject*
 operationNew(PyObject* /*arg*/)
 {
     OperationObject* self = PyObject_New(OperationObject, &OperationType);
-    if (self == NULL)
+    if(!self)
     {
-        return NULL;
+        return 0;
     }
     self->op = 0;
     return self;
@@ -143,20 +148,23 @@ operationInit(OperationObject* self, PyObject* args, PyObject* /*kwds*/)
 {
     char* name;
     PyObject* modeType = lookupType("Ice.OperationMode");
-    assert(modeType != NULL);
+    assert(modeType);
     PyObject* mode;
+    PyObject* sendMode;
     int amd;
+    PyObject* meta;
     PyObject* inParams;
     PyObject* outParams;
     PyObject* returnType;
     PyObject* exceptions;
-    if(!PyArg_ParseTuple(args, STRCAST("sO!iO!O!OO!"), &name, modeType, &mode, &amd, &PyTuple_Type, &inParams,
-                         &PyTuple_Type, &outParams, &returnType, &PyTuple_Type, &exceptions))
+    if(!PyArg_ParseTuple(args, STRCAST("sO!O!iO!O!O!OO!"), &name, modeType, &mode, modeType, &sendMode, &amd,
+                         &PyTuple_Type, &meta, &PyTuple_Type, &inParams, &PyTuple_Type, &outParams, &returnType,
+                         &PyTuple_Type, &exceptions))
     {
         return -1;
     }
 
-    OperationIPtr op = new OperationI(name, mode, amd, inParams, outParams, returnType, exceptions);
+    OperationIPtr op = new OperationI(name, mode, sendMode, amd, meta, inParams, outParams, returnType, exceptions);
     self->op = new OperationPtr(op);
 
     return 0;
@@ -183,13 +191,13 @@ operationInvoke(OperationObject* self, PyObject* args)
     PyObject* ctx;
     if(!PyArg_ParseTuple(args, STRCAST("O!O!O"), &ProxyType, &pyProxy, &PyTuple_Type, &opArgs, &ctx))
     {
-        return NULL;
+        return 0;
     }
 
     if(ctx != Py_None && !PyDict_Check(ctx))
     {
         PyErr_Format(PyExc_ValueError, STRCAST("context argument must be None or a dictionary"));
-        return NULL;
+        return 0;
     }
 
     Ice::ObjectPrx prx = getProxy(pyProxy);
@@ -210,13 +218,13 @@ operationInvokeAsync(OperationObject* self, PyObject* args)
     PyObject* ctx;
     if(!PyArg_ParseTuple(args, STRCAST("O!OO!O"), &ProxyType, &pyProxy, &cb, &PyTuple_Type, &opArgs, &ctx))
     {
-        return NULL;
+        return 0;
     }
 
     if(ctx != Py_None && !PyDict_Check(ctx))
     {
         PyErr_Format(PyExc_ValueError, STRCAST("context argument must be None or a dictionary"));
-        return NULL;
+        return 0;
     }
 
     Ice::ObjectPrx prx = getProxy(pyProxy);
@@ -234,7 +242,7 @@ operationDeprecate(OperationObject* self, PyObject* args)
     char* msg;
     if(!PyArg_ParseTuple(args, STRCAST("s"), &msg))
     {
-        return NULL;
+        return 0;
     }
 
     assert(self->op);
@@ -251,9 +259,9 @@ static AMDCallbackObject*
 amdCallbackNew(PyObject* /*arg*/)
 {
     AMDCallbackObject* self = PyObject_New(AMDCallbackObject, &AMDCallbackType);
-    if (self == NULL)
+    if(!self)
     {
-        return NULL;
+        return 0;
     }
     self->op = 0;
     self->communicator = 0;
@@ -309,14 +317,14 @@ amdCallbackIceException(AMDCallbackObject* self, PyObject* args)
     PyObject* ex;
     if(!PyArg_ParseTuple(args, STRCAST("O"), &ex))
     {
-        return NULL;
+        return 0;
     }
 
     try
     {
         assert(self->op);
-	PyException pye(ex); // No traceback information available.
-	(*self->op)->sendException(*self->cb, pye, *self->communicator);
+        PyException pye(ex); // No traceback information available.
+        (*self->op)->sendException(*self->cb, pye, *self->communicator);
     }
     catch(const Ice::Exception& ex)
     {
@@ -384,17 +392,35 @@ IcePy::AMICallback::ice_exception(const Ice::Exception& ex)
     AdoptThread adoptThread; // Ensure the current thread is able to call into Python.
 
     PyObjectHandle exh = convertException(ex);
-    assert(exh.get() != NULL);
+    assert(exh.get());
     _op->responseAsyncException(_callback, exh.get());
 }
 
 //
 // OperationI implementation.
 //
-IcePy::OperationI::OperationI(const char* name, PyObject* mode, int amd, PyObject* inParams, PyObject* outParams,
-                              PyObject* returnType, PyObject* exceptions)
+IcePy::OperationI::OperationI(const char* name, PyObject* mode, PyObject* sendMode, int amd, PyObject* meta,
+                              PyObject* inParams, PyObject* outParams, PyObject* returnType, PyObject* exceptions)
 {
     _name = name;
+
+    //
+    // mode
+    //
+    PyObjectHandle modeValue = PyObject_GetAttrString(mode, STRCAST("value"));
+    assert(PyInt_Check(modeValue.get()));
+    _mode = (Ice::OperationMode)static_cast<int>(PyInt_AS_LONG(modeValue.get()));
+
+    //
+    // sendMode
+    //
+    PyObjectHandle sendModeValue = PyObject_GetAttrString(sendMode, STRCAST("value"));
+    assert(PyInt_Check(sendModeValue.get()));
+    _sendMode = (Ice::OperationMode)static_cast<int>(PyInt_AS_LONG(sendModeValue.get()));
+
+    //
+    // amd
+    //
     _amd = amd ? true : false;
     if(_amd)
     {
@@ -406,45 +432,25 @@ IcePy::OperationI::OperationI(const char* name, PyObject* mode, int amd, PyObjec
     }
 
     //
-    // mode
+    // metaData
     //
-    PyObjectHandle modeValue = PyObject_GetAttrString(mode, STRCAST("value"));
-    assert(PyInt_Check(modeValue.get()));
-    _mode = (Ice::OperationMode)static_cast<int>(PyInt_AS_LONG(modeValue.get()));
+#ifndef NDEBUG
+    bool b =
+#endif
+    tupleToStringSeq(meta, _metaData);
+    assert(b);
 
     Py_ssize_t i, sz;
 
     //
     // inParams
     //
-    _sendsClasses = false;
-    sz = PyTuple_GET_SIZE(inParams);
-    for(i = 0; i < sz; ++i)
-    {
-        ParamInfoPtr param = new ParamInfo;
-        param->type = getType(PyTuple_GET_ITEM(inParams, i));
-        _inParams.push_back(param);
-        if(!_sendsClasses)
-        {
-            _sendsClasses = param->type->usesClasses();
-        }
-    }
+    convertParams(inParams, _inParams, _sendsClasses);
 
     //
     // outParams
     //
-    _returnsClasses = false;
-    sz = PyTuple_GET_SIZE(outParams);
-    for(i = 0; i < sz; ++i)
-    {
-        ParamInfoPtr param = new ParamInfo;
-        param->type = getType(PyTuple_GET_ITEM(outParams, i));
-        _outParams.push_back(param);
-        if(!_returnsClasses)
-        {
-            _returnsClasses = param->type->usesClasses();
-        }
-    }
+    convertParams(outParams, _outParams, _returnsClasses);
 
     //
     // returnType
@@ -480,18 +486,18 @@ IcePy::OperationI::invoke(const Ice::ObjectPrx& proxy, PyObject* args, PyObject*
     Ice::ByteSeq params;
     if(!prepareRequest(communicator, args, false, params))
     {
-        return NULL;
+        return 0;
     }
 
     if(!_deprecateMessage.empty())
     {
-	PyErr_Warn(PyExc_DeprecationWarning, const_cast<char*>(_deprecateMessage.c_str()));
-	_deprecateMessage.clear(); // Only show the warning once.
+        PyErr_Warn(PyExc_DeprecationWarning, const_cast<char*>(_deprecateMessage.c_str()));
+        _deprecateMessage.clear(); // Only show the warning once.
     }
 
     try
     {
-	checkTwowayOnly(proxy);
+        checkTwowayOnly(proxy);
 
         //
         // Invoke the operation.
@@ -506,21 +512,21 @@ IcePy::OperationI::invoke(const Ice::ObjectPrx& proxy, PyObject* args, PyObject*
                 if(!PyDict_Check(pyctx))
                 {
                     PyErr_Format(PyExc_ValueError, STRCAST("context argument must be None or a dictionary"));
-                    return NULL;
+                    return 0;
                 }
 
                 if(!dictionaryToContext(pyctx, ctx))
                 {
-                    return NULL;
+                    return 0;
                 }
 
                 AllowThreads allowThreads; // Release Python's global interpreter lock during remote invocations.
-                status = proxy->ice_invoke(_name, _mode, params, result, ctx);
+                status = proxy->ice_invoke(_name, _sendMode, params, result, ctx);
             }
             else
             {
                 AllowThreads allowThreads; // Release Python's global interpreter lock during remote invocations.
-                status = proxy->ice_invoke(_name, _mode, params, result);
+                status = proxy->ice_invoke(_name, _sendMode, params, result);
             }
         }
 
@@ -539,8 +545,8 @@ IcePy::OperationI::invoke(const Ice::ObjectPrx& proxy, PyObject* args, PyObject*
                 //
                 // Set the Python exception.
                 //
-		setPythonException(ex.get());
-                return NULL;
+                setPythonException(ex.get());
+                return 0;
             }
             else if(_outParams.size() > 0 || _returnType)
             {
@@ -549,9 +555,9 @@ IcePy::OperationI::invoke(const Ice::ObjectPrx& proxy, PyObject* args, PyObject*
                 // in a tuple of the form (result, outParam1, ...). Otherwise just return the value.
                 //
                 PyObjectHandle results = unmarshalResults(result, communicator);
-                if(results.get() == NULL)
+                if(!results.get())
                 {
-                    return NULL;
+                    return 0;
                 }
 
                 if(PyTuple_GET_SIZE(results.get()) > 1)
@@ -569,12 +575,12 @@ IcePy::OperationI::invoke(const Ice::ObjectPrx& proxy, PyObject* args, PyObject*
     }
     catch(const AbortMarshaling&)
     {
-        return NULL;
+        return 0;
     }
     catch(const Ice::Exception& ex)
     {
         setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     Py_INCREF(Py_None);
@@ -592,19 +598,19 @@ IcePy::OperationI::invokeAsync(const Ice::ObjectPrx& proxy, PyObject* callback,
     Ice::ByteSeq params;
     if(!prepareRequest(communicator, args, true, params))
     {
-        return NULL;
+        return 0;
     }
 
     if(!_deprecateMessage.empty())
     {
-	PyErr_Warn(PyExc_DeprecationWarning, const_cast<char*>(_deprecateMessage.c_str()));
-	_deprecateMessage.clear(); // Only show the warning once.
+        PyErr_Warn(PyExc_DeprecationWarning, const_cast<char*>(_deprecateMessage.c_str()));
+        _deprecateMessage.clear(); // Only show the warning once.
     }
 
     Ice::AMI_Object_ice_invokePtr cb = new AMICallback(this, communicator, callback);
     try
     {
-	checkTwowayOnly(proxy);
+        checkTwowayOnly(proxy);
 
         //
         // Invoke the operation asynchronously.
@@ -616,26 +622,26 @@ IcePy::OperationI::invokeAsync(const Ice::ObjectPrx& proxy, PyObject* callback,
             if(!PyDict_Check(pyctx))
             {
                 PyErr_Format(PyExc_ValueError, STRCAST("context argument must be None or a dictionary"));
-                return NULL;
+                return 0;
             }
 
             if(!dictionaryToContext(pyctx, ctx))
             {
-                return NULL;
+                return 0;
             }
 
             AllowThreads allowThreads; // Release Python's global interpreter lock during remote invocations.
-            proxy->ice_invoke_async(cb, _name, _mode, params, ctx);
+            proxy->ice_invoke_async(cb, _name, _sendMode, params, ctx);
         }
         else
         {
             AllowThreads allowThreads; // Release Python's global interpreter lock during remote invocations.
-            proxy->ice_invoke_async(cb, _name, _mode, params);
+            proxy->ice_invoke_async(cb, _name, _sendMode, params);
         }
     }
     catch(const Ice::Exception& ex)
     {
-	cb->ice_exception(ex);
+        cb->ice_exception(ex);
     }
 
     Py_INCREF(Py_None);
@@ -647,14 +653,20 @@ IcePy::OperationI::deprecate(const string& msg)
 {
     if(!msg.empty())
     {
-	_deprecateMessage = msg;
+        _deprecateMessage = msg;
     }
     else
     {
-	_deprecateMessage = "operation " + _name + " is deprecated";
+        _deprecateMessage = "operation " + _name + " is deprecated";
     }
 }
 
+Ice::OperationMode
+IcePy::OperationI::mode() const
+{
+    return _mode;
+}
+
 void
 IcePy::OperationI::dispatch(PyObject* servant, const Ice::AMD_Object_ice_invokePtr& cb,
                             const vector<Ice::Byte>& inBytes, const Ice::Current& current)
@@ -674,7 +686,7 @@ IcePy::OperationI::dispatch(PyObject* servant, const Ice::AMD_Object_ice_invokeP
     }
 
     PyObjectHandle args = PyTuple_New(count);
-    if(args.get() == NULL)
+    if(!args.get())
     {
         throwPythonException();
     }
@@ -687,8 +699,8 @@ IcePy::OperationI::dispatch(PyObject* servant, const Ice::AMD_Object_ice_invokeP
             Py_ssize_t i = start;
             for(ParamInfoList::iterator p = _inParams.begin(); p != _inParams.end(); ++p, ++i)
             {
-		void* closure = reinterpret_cast<void*>(i);
-                (*p)->type->unmarshal(is, *p, args.get(), closure);
+                void* closure = reinterpret_cast<void*>(i);
+                (*p)->type->unmarshal(is, *p, args.get(), closure, &(*p)->metaData);
             }
             if(_sendsClasses)
             {
@@ -716,8 +728,8 @@ IcePy::OperationI::dispatch(PyObject* servant, const Ice::AMD_Object_ice_invokeP
         //
         // Create the callback object and pass it as the first argument.
         //
-        AMDCallbackObject* obj = amdCallbackNew(NULL);
-        if(obj == NULL)
+        AMDCallbackObject* obj = amdCallbackNew(0);
+        if(!obj)
         {
             throwPythonException();
         }
@@ -735,7 +747,7 @@ IcePy::OperationI::dispatch(PyObject* servant, const Ice::AMD_Object_ice_invokeP
     // Dispatch the operation. Use _dispatchName here, not current.operation.
     //
     PyObjectHandle method = PyObject_GetAttrString(servant, const_cast<char*>(_dispatchName.c_str()));
-    if(method.get() == NULL)
+    if(!method.get())
     {
         ostringstream ostr;
         ostr << "servant for identity " << communicator->identityToString(current.id) << " does not define operation `"
@@ -747,15 +759,15 @@ IcePy::OperationI::dispatch(PyObject* servant, const Ice::AMD_Object_ice_invokeP
         throw ex;
     }
 
-    PyObjectHandle result = PyObject_Call(method.get(), args.get(), NULL);
+    PyObjectHandle result = PyObject_Call(method.get(), args.get(), 0);
 
     //
     // Check for exceptions.
     //
     if(PyErr_Occurred())
     {
-	PyException ex; // Retrieve it before another Python API call clears it.
-	sendException(cb, ex, communicator);
+        PyException ex; // Retrieve it before another Python API call clears it.
+        sendException(cb, ex, communicator);
         return;
     }
 
@@ -773,29 +785,29 @@ IcePy::OperationI::responseAsync(PyObject* callback, bool ok, const vector<Ice::
     {
         if(ok)
         {
-	    //
-	    // Unmarshal the results.
-	    //
-	    PyObjectHandle args;
-	    try
-	    {
-		args = unmarshalResults(results, communicator);
-		if(args.get() == NULL)
-		{
-		    assert(PyErr_Occurred());
-		    PyErr_Print();
-		    return;
-		}
-	    }
-	    catch(const Ice::Exception& ex)
-	    {
-		PyObjectHandle h = convertException(ex);
-		responseAsyncException(callback, h.get());
-		return;
-	    }
+            //
+            // Unmarshal the results.
+            //
+            PyObjectHandle args;
+            try
+            {
+                args = unmarshalResults(results, communicator);
+                if(!args.get())
+                {
+                    assert(PyErr_Occurred());
+                    PyErr_Print();
+                    return;
+                }
+            }
+            catch(const Ice::Exception& ex)
+            {
+                PyObjectHandle h = convertException(ex);
+                responseAsyncException(callback, h.get());
+                return;
+            }
 
             PyObjectHandle method = PyObject_GetAttrString(callback, STRCAST("ice_response"));
-            if(method.get() == NULL)
+            if(!method.get())
             {
                 ostringstream ostr;
                 ostr << "AMI callback object for operation `" << _name << "' does not define ice_response()";
@@ -804,7 +816,7 @@ IcePy::OperationI::responseAsync(PyObject* callback, bool ok, const vector<Ice::
             }
             else
             {
-                PyObjectHandle tmp = PyObject_Call(method.get(), args.get(), NULL);
+                PyObjectHandle tmp = PyObject_Call(method.get(), args.get(), 0);
                 if(PyErr_Occurred())
                 {
                     PyErr_Print();
@@ -835,7 +847,7 @@ void
 IcePy::OperationI::responseAsyncException(PyObject* callback, PyObject* ex)
 {
     PyObjectHandle method = PyObject_GetAttrString(callback, STRCAST("ice_exception"));
-    if(method.get() == NULL)
+    if(!method.get())
     {
         ostringstream ostr;
         ostr << "AMI callback object for operation `" << _name << "' does not define ice_exception()";
@@ -845,7 +857,7 @@ IcePy::OperationI::responseAsyncException(PyObject* callback, PyObject* ex)
     else
     {
         PyObjectHandle args = Py_BuildValue(STRCAST("(O)"), ex);
-        PyObjectHandle tmp = PyObject_Call(method.get(), args.get(), NULL);
+        PyObjectHandle tmp = PyObject_Call(method.get(), args.get(), 0);
         if(PyErr_Occurred())
         {
             PyErr_Print();
@@ -903,7 +915,7 @@ IcePy::OperationI::sendResponse(const Ice::AMD_Object_ice_invokePtr& cb, PyObjec
                 PyErr_Warn(PyExc_RuntimeWarning, const_cast<char*>(str.c_str()));
                 throw Ice::MarshalException(__FILE__, __LINE__);
             }
-            (*p)->type->marshal(arg, os, &objectMap);
+            (*p)->type->marshal(arg, os, &objectMap, &(*p)->metaData);
         }
 
         if(_returnType)
@@ -926,7 +938,7 @@ IcePy::OperationI::sendResponse(const Ice::AMD_Object_ice_invokePtr& cb, PyObjec
                 PyErr_Warn(PyExc_RuntimeWarning, const_cast<char*>(str.c_str()));
                 throw Ice::MarshalException(__FILE__, __LINE__);
             }
-            _returnType->type->marshal(res, os, &objectMap);
+            _returnType->type->marshal(res, os, &objectMap, &_metaData);
         }
 
         if(_returnsClasses)
@@ -956,25 +968,25 @@ IcePy::OperationI::sendException(const Ice::AMD_Object_ice_invokePtr& cb, PyExce
         // However, we have no way to pass this exception to the interpreter,
         // so we act on it directly.
         //
-	if(PyObject_IsInstance(ex.ex.get(), PyExc_SystemExit))
+        if(PyObject_IsInstance(ex.ex.get(), PyExc_SystemExit))
         {
             handleSystemExit(ex.ex.get()); // Does not return.
         }
 
         PyObject* userExceptionType = lookupType("Ice.UserException");
 
-	if(PyObject_IsInstance(ex.ex.get(), userExceptionType))
+        if(PyObject_IsInstance(ex.ex.get(), userExceptionType))
         {
             //
             // Get the exception's type and verify that it is legal to be thrown from this operation.
             //
             PyObjectHandle iceType = PyObject_GetAttrString(ex.ex.get(), STRCAST("ice_type"));
-            assert(iceType.get() != NULL);
+            assert(iceType.get());
             ExceptionInfoPtr info = ExceptionInfoPtr::dynamicCast(getException(iceType.get()));
             assert(info);
             if(!validateException(ex.ex.get()))
             {
-		ex.raise(); // Raises UnknownUserException.
+                ex.raise(); // Raises UnknownUserException.
             }
             else
             {
@@ -994,7 +1006,7 @@ IcePy::OperationI::sendException(const Ice::AMD_Object_ice_invokePtr& cb, PyExce
         }
         else
         {
-	    ex.raise();
+            ex.raise();
         }
     }
     catch(const AbortMarshaling&)
@@ -1018,7 +1030,7 @@ IcePy::OperationI::prepareRequest(const Ice::CommunicatorPtr& communicator, PyOb
     {
         string fixedName = fixIdent(_name);
         PyErr_Format(PyExc_RuntimeError, STRCAST("%s expects %d in parameters"), fixedName.c_str(),
-		     static_cast<int>(paramCount));
+                     static_cast<int>(paramCount));
         return false;
     }
 
@@ -1051,7 +1063,7 @@ IcePy::OperationI::prepareRequest(const Ice::CommunicatorPtr& communicator, PyOb
                                  async ? i + 2 : i + 1, const_cast<char*>(opName.c_str()));
                     return false;
                 }
-                (*p)->type->marshal(arg, os, &objectMap);
+                (*p)->type->marshal(arg, os, &objectMap, &(*p)->metaData);
             }
 
             if(_sendsClasses)
@@ -1082,7 +1094,7 @@ IcePy::OperationI::unmarshalResults(const vector<Ice::Byte>& bytes, const Ice::C
     Py_ssize_t numResults = static_cast<Py_ssize_t>(_outParams.size()) + i;
 
     PyObjectHandle results = PyTuple_New(numResults);
-    if(results.get() != NULL && numResults > 0)
+    if(results.get() && numResults > 0)
     {
         //
         // Unmarshal the results. If there is more than one value to be returned, then return them
@@ -1091,13 +1103,13 @@ IcePy::OperationI::unmarshalResults(const vector<Ice::Byte>& bytes, const Ice::C
         Ice::InputStreamPtr is = Ice::createInputStream(communicator, bytes);
         for(ParamInfoList::iterator p = _outParams.begin(); p != _outParams.end(); ++p, ++i)
         {
-	    void* closure = reinterpret_cast<void*>(i);
-            (*p)->type->unmarshal(is, *p, results.get(), closure);
+            void* closure = reinterpret_cast<void*>(i);
+            (*p)->type->unmarshal(is, *p, results.get(), closure, &(*p)->metaData);
         }
 
         if(_returnType)
         {
-            _returnType->type->unmarshal(is, _returnType, results.get(), 0);
+            _returnType->type->unmarshal(is, _returnType, results.get(), 0, &_metaData);
         }
 
         if(_returnsClasses)
@@ -1134,8 +1146,8 @@ IcePy::OperationI::unmarshalException(const vector<Ice::Byte>& bytes, const Ice:
             }
             else
             {
-		PyException pye(ex.get()); // No traceback information available.
-		pye.raise();
+                PyException pye(ex.get()); // No traceback information available.
+                pye.raise();
             }
         }
         else
@@ -1173,30 +1185,66 @@ IcePy::OperationI::checkTwowayOnly(const Ice::ObjectPrx& proxy) const
 {
     if((_returnType != 0 || !_outParams.empty()) && !proxy->ice_isTwoway())
     {
-	Ice::TwowayOnlyException ex(__FILE__, __LINE__);
-	ex.operation = _name;
-	throw ex;
+        Ice::TwowayOnlyException ex(__FILE__, __LINE__);
+        ex.operation = _name;
+        throw ex;
+    }
+}
+
+void
+IcePy::OperationI::convertParams(PyObject* p, ParamInfoList& params, bool& usesClasses)
+{
+    usesClasses = false;
+    int sz = static_cast<int>(PyTuple_GET_SIZE(p));
+    for(int i = 0; i < sz; ++i)
+    {
+        PyObject* item = PyTuple_GET_ITEM(p, i);
+        assert(PyTuple_Check(item));
+        assert(PyTuple_GET_SIZE(item) == 2);
+
+        ParamInfoPtr param = new ParamInfo;
+
+        //
+        // metaData
+        //
+        PyObject* meta = PyTuple_GET_ITEM(item, 0);
+        assert(PyTuple_Check(meta));
+#ifndef NDEBUG
+        bool b =
+#endif
+        tupleToStringSeq(meta, param->metaData);
+        assert(b);
+
+        //
+        // type
+        //
+        param->type = getType(PyTuple_GET_ITEM(item, 1));
+        params.push_back(param);
+        if(!usesClasses)
+        {
+            usesClasses = param->type->usesClasses();
+        }
     }
 }
 
 static PyMethodDef OperationMethods[] =
 {
-    { STRCAST("invoke"), (PyCFunction)operationInvoke, METH_VARARGS,
+    { STRCAST("invoke"), reinterpret_cast<PyCFunction>(operationInvoke), METH_VARARGS,
       PyDoc_STR(STRCAST("internal function")) },
-    { STRCAST("invokeAsync"), (PyCFunction)operationInvokeAsync, METH_VARARGS,
+    { STRCAST("invokeAsync"), reinterpret_cast<PyCFunction>(operationInvokeAsync), METH_VARARGS,
       PyDoc_STR(STRCAST("internal function")) },
-    { STRCAST("deprecate"), (PyCFunction)operationDeprecate, METH_VARARGS,
+    { STRCAST("deprecate"), reinterpret_cast<PyCFunction>(operationDeprecate), METH_VARARGS,
       PyDoc_STR(STRCAST("internal function")) },
-    { NULL, NULL} /* sentinel */
+    { 0, 0 } /* sentinel */
 };
 
 static PyMethodDef AMDCallbackMethods[] =
 {
-    { STRCAST("ice_response"), (PyCFunction)amdCallbackIceResponse, METH_VARARGS,
+    { STRCAST("ice_response"), reinterpret_cast<PyCFunction>(amdCallbackIceResponse), METH_VARARGS,
       PyDoc_STR(STRCAST("internal function")) },
-    { STRCAST("ice_exception"), (PyCFunction)amdCallbackIceException, METH_VARARGS,
+    { STRCAST("ice_exception"), reinterpret_cast<PyCFunction>(amdCallbackIceException), METH_VARARGS,
       PyDoc_STR(STRCAST("internal function")) },
-    { NULL, NULL} /* sentinel */
+    { 0, 0 } /* sentinel */
 };
 
 namespace IcePy
@@ -1206,13 +1254,13 @@ PyTypeObject OperationType =
 {
     /* The ob_type field must be initialized in the module init function
      * to be portable to Windows without using C++. */
-    PyObject_HEAD_INIT(NULL)
+    PyObject_HEAD_INIT(0)
     0,                               /* ob_size */
     STRCAST("IcePy.Operation"),      /* tp_name */
     sizeof(OperationObject),         /* tp_basicsize */
     0,                               /* tp_itemsize */
     /* methods */
-    (destructor)operationDealloc,    /* tp_dealloc */
+    reinterpret_cast<destructor>(operationDealloc), /* tp_dealloc */
     0,                               /* tp_print */
     0,                               /* tp_getattr */
     0,                               /* tp_setattr */
@@ -1243,9 +1291,9 @@ PyTypeObject OperationType =
     0,                               /* tp_descr_get */
     0,                               /* tp_descr_set */
     0,                               /* tp_dictoffset */
-    (initproc)operationInit,         /* tp_init */
+    reinterpret_cast<initproc>(operationInit), /* tp_init */
     0,                               /* tp_alloc */
-    (newfunc)operationNew,           /* tp_new */
+    reinterpret_cast<newfunc>(operationNew), /* tp_new */
     0,                               /* tp_free */
     0,                               /* tp_is_gc */
 };
@@ -1254,13 +1302,13 @@ PyTypeObject AMDCallbackType =
 {
     /* The ob_type field must be initialized in the module init function
      * to be portable to Windows without using C++. */
-    PyObject_HEAD_INIT(NULL)
+    PyObject_HEAD_INIT(0)
     0,                               /* ob_size */
     STRCAST("IcePy.AMDCallback"),    /* tp_name */
     sizeof(AMDCallbackObject),       /* tp_basicsize */
     0,                               /* tp_itemsize */
     /* methods */
-    (destructor)amdCallbackDealloc,  /* tp_dealloc */
+    reinterpret_cast<destructor>(amdCallbackDealloc), /* tp_dealloc */
     0,                               /* tp_print */
     0,                               /* tp_getattr */
     0,                               /* tp_setattr */
@@ -1293,7 +1341,7 @@ PyTypeObject AMDCallbackType =
     0,                               /* tp_dictoffset */
     0,                               /* tp_init */
     0,                               /* tp_alloc */
-    (newfunc)amdCallbackNew,         /* tp_new */
+    reinterpret_cast<newfunc>(amdCallbackNew), /* tp_new */
     0,                               /* tp_free */
     0,                               /* tp_is_gc */
 };
@@ -1307,7 +1355,8 @@ IcePy::initOperation(PyObject* module)
     {
         return false;
     }
-    if(PyModule_AddObject(module, STRCAST("Operation"), (PyObject*)&OperationType) < 0)
+    PyTypeObject* opType = &OperationType; // Necessary to prevent GCC's strict-alias warnings.
+    if(PyModule_AddObject(module, STRCAST("Operation"), reinterpret_cast<PyObject*>(opType)) < 0)
     {
         return false;
     }
@@ -1316,7 +1365,8 @@ IcePy::initOperation(PyObject* module)
     {
         return false;
     }
-    if(PyModule_AddObject(module, STRCAST("AMDCallback"), (PyObject*)&AMDCallbackType) < 0)
+    PyTypeObject* cbType = &AMDCallbackType; // Necessary to prevent GCC's strict-alias warnings.
+    if(PyModule_AddObject(module, STRCAST("AMDCallback"), reinterpret_cast<PyObject*>(cbType)) < 0)
     {
         return false;
     }
@@ -1327,7 +1377,7 @@ IcePy::initOperation(PyObject* module)
 IcePy::OperationPtr
 IcePy::getOperation(PyObject* p)
 {
-    assert(PyObject_IsInstance(p, (PyObject*)&OperationType) == 1);
-    OperationObject* obj = (OperationObject*)p;
+    assert(PyObject_IsInstance(p, reinterpret_cast<PyObject*>(&OperationType)) == 1);
+    OperationObject* obj = reinterpret_cast<OperationObject*>(p);
     return *obj->op;
 }
