@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -67,7 +67,7 @@ PyObject*
 IcePy::PyObjectHandle::release()
 {
     PyObject* result = _p;
-    _p = NULL;
+    _p = 0;
     return result;
 }
 
@@ -101,54 +101,54 @@ IcePy::PyException::raise()
 
     if(PyObject_IsInstance(ex.get(), userExceptionType))
     {
-	Ice::UnknownUserException e(__FILE__, __LINE__);
-	string tb = getTraceback();
-	if(!tb.empty())
-	{
-	    e.unknown = tb;
-	}
-	else
-	{
-	    PyObjectHandle name = PyObject_CallMethod(ex.get(), STRCAST("ice_name"), NULL);
-	    PyErr_Clear();
-	    if(name.get() == NULL)
-	    {
-		e.unknown = getTypeName();
-	    }
-	    else
-	    {
-		e.unknown = PyString_AS_STRING(name.get());
-	    }
-	}
-	throw e;
+        Ice::UnknownUserException e(__FILE__, __LINE__);
+        string tb = getTraceback();
+        if(!tb.empty())
+        {
+            e.unknown = tb;
+        }
+        else
+        {
+            PyObjectHandle name = PyObject_CallMethod(ex.get(), STRCAST("ice_name"), 0);
+            PyErr_Clear();
+            if(!name.get())
+            {
+                e.unknown = getTypeName();
+            }
+            else
+            {
+                e.unknown = PyString_AS_STRING(name.get());
+            }
+        }
+        throw e;
     }
     else if(PyObject_IsInstance(ex.get(), localExceptionType))
     {
-	raiseLocalException();
+        raiseLocalException();
     }
     else
     {
-	Ice::UnknownException e(__FILE__, __LINE__);
-	string tb = getTraceback();
-	if(!tb.empty())
-	{
-	    e.unknown = tb;
-	}
-	else
-	{
-	    ostringstream ostr;
+        Ice::UnknownException e(__FILE__, __LINE__);
+        string tb = getTraceback();
+        if(!tb.empty())
+        {
+            e.unknown = tb;
+        }
+        else
+        {
+            ostringstream ostr;
 
-	    ostr << getTypeName();
+            ostr << getTypeName();
 
-	    IcePy::PyObjectHandle msg = PyObject_Str(ex.get());
-	    if(msg.get() != NULL && strlen(PyString_AsString(msg.get())) > 0)
-	    {
-		ostr << ": " << PyString_AsString(msg.get());
-	    }
+            IcePy::PyObjectHandle msg = PyObject_Str(ex.get());
+            if(msg.get() && strlen(PyString_AsString(msg.get())) > 0)
+            {
+                ostr << ": " << PyString_AsString(msg.get());
+            }
 
-	    e.unknown = ostr.str();
-	}
-	throw e;
+            e.unknown = ostr.str();
+        }
+        throw e;
     }
 }
 
@@ -159,87 +159,79 @@ IcePy::PyException::raiseLocalException()
 
     try
     {
-	if(typeName == "Ice.ObjectNotExistException")
-	{
-	    throw Ice::ObjectNotExistException(__FILE__, __LINE__);
-	}
-	else if(typeName == "Ice.OperationNotExistException")
-	{
-	    throw Ice::OperationNotExistException(__FILE__, __LINE__);
-	}
-	else if(typeName == "Ice.FacetNotExistException")
-	{
-	    throw Ice::FacetNotExistException(__FILE__, __LINE__);
-	}
-	else if(typeName == "Ice.RequestFailedException")
-	{
-	    throw Ice::RequestFailedException(__FILE__, __LINE__);
-	}
+        if(typeName == "Ice.ObjectNotExistException")
+        {
+            throw Ice::ObjectNotExistException(__FILE__, __LINE__);
+        }
+        else if(typeName == "Ice.OperationNotExistException")
+        {
+            throw Ice::OperationNotExistException(__FILE__, __LINE__);
+        }
+        else if(typeName == "Ice.FacetNotExistException")
+        {
+            throw Ice::FacetNotExistException(__FILE__, __LINE__);
+        }
+        else if(typeName == "Ice.RequestFailedException")
+        {
+            throw Ice::RequestFailedException(__FILE__, __LINE__);
+        }
     }
     catch(Ice::RequestFailedException& e)
     {
-	IcePy::PyObjectHandle member;
-	member = PyObject_GetAttrString(ex.get(), STRCAST("id"));
-	if(member.get() != NULL && IcePy::checkIdentity(member.get()))
-	{
-	    IcePy::getIdentity(member.get(), e.id);
-	}
-	member = PyObject_GetAttrString(ex.get(), STRCAST("facet"));
-	if(member.get() != NULL && PyString_Check(member.get()))
-	{
-	    e.facet = PyString_AS_STRING(member.get());
-	}
-	member = PyObject_GetAttrString(ex.get(), STRCAST("operation"));
-	if(member.get() != NULL && PyString_Check(member.get()))
-	{
-	    e.operation = PyString_AS_STRING(member.get());
-	}
-	throw e;
+        IcePy::PyObjectHandle member;
+        member = PyObject_GetAttrString(ex.get(), STRCAST("id"));
+        if(member.get() && IcePy::checkIdentity(member.get()))
+        {
+            IcePy::getIdentity(member.get(), e.id);
+        }
+        member = PyObject_GetAttrString(ex.get(), STRCAST("facet"));
+        if(member.get() && PyString_Check(member.get()))
+        {
+            e.facet = PyString_AS_STRING(member.get());
+        }
+        member = PyObject_GetAttrString(ex.get(), STRCAST("operation"));
+        if(member.get() && PyString_Check(member.get()))
+        {
+            e.operation = PyString_AS_STRING(member.get());
+        }
+        throw;
     }
 
     try
     {
-	if(typeName == "Ice.UnknownLocalException")
-	{
-	    throw Ice::UnknownLocalException(__FILE__, __LINE__);
-	}
-	else if(typeName == "Ice.UnknownUserException")
-	{
-	    throw Ice::UnknownUserException(__FILE__, __LINE__);
-	}
-	else if(typeName == "Ice.UnknownException")
-	{
-	    throw Ice::UnknownException(__FILE__, __LINE__);
-	}
+        if(typeName == "Ice.UnknownLocalException")
+        {
+            throw Ice::UnknownLocalException(__FILE__, __LINE__);
+        }
+        else if(typeName == "Ice.UnknownUserException")
+        {
+            throw Ice::UnknownUserException(__FILE__, __LINE__);
+        }
+        else if(typeName == "Ice.UnknownException")
+        {
+            throw Ice::UnknownException(__FILE__, __LINE__);
+        }
     }
     catch(Ice::UnknownException& e)
     {
-	string tb = getTraceback();
-	if(!tb.empty())
-	{
-	    e.unknown = tb;
-	}
-	else
-	{
-	    IcePy::PyObjectHandle member;
-	    member = PyObject_GetAttrString(ex.get(), STRCAST("unknown"));
-	    if(member.get() != NULL && PyString_Check(member.get()) && strlen(PyString_AS_STRING(member.get())) > 0)
-	    {
-		e.unknown = PyString_AS_STRING(member.get());
-	    }
-	}
-	throw e;
+        IcePy::PyObjectHandle member;
+        member = PyObject_GetAttrString(ex.get(), STRCAST("unknown"));
+        if(member.get() && PyString_Check(member.get()) && strlen(PyString_AS_STRING(member.get())) > 0)
+        {
+            e.unknown = PyString_AS_STRING(member.get());
+        }
+        throw;
     }
 
     Ice::UnknownLocalException e(__FILE__, __LINE__);
     string tb = getTraceback();
     if(!tb.empty())
     {
-	e.unknown = tb;
+        e.unknown = tb;
     }
     else
     {
-	e.unknown = typeName;
+        e.unknown = typeName;
     }
     throw e;
 }
@@ -247,9 +239,9 @@ IcePy::PyException::raiseLocalException()
 string
 IcePy::PyException::getTraceback()
 {
-    if(_tb.get() == NULL)
+    if(!_tb.get())
     {
-	return string();
+        return string();
     }
 
     //
@@ -260,17 +252,17 @@ IcePy::PyException::getTraceback()
     //
     PyObjectHandle str = PyString_FromString("traceback");
     PyObjectHandle mod = PyImport_Import(str.get());
-    assert(mod.get() != NULL); // Unable to import traceback module - Python installation error?
+    assert(mod.get()); // Unable to import traceback module - Python installation error?
     PyObject* d = PyModule_GetDict(mod.get());
     PyObject* func = PyDict_GetItemString(d, "format_exception");
-    assert(func != NULL); // traceback.format_exception must be present.
+    assert(func); // traceback.format_exception must be present.
     PyObjectHandle args = Py_BuildValue("(OOO)", _type.get(), ex.get(), _tb.get());
     PyObjectHandle list = PyObject_CallObject(func, args.get());
 
     string result;
     for(Py_ssize_t i = 0; i < PyList_GET_SIZE(list.get()); ++i)
     {
-	result += PyString_AsString(PyList_GetItem(list.get(), i));
+        result += PyString_AsString(PyList_GetItem(list.get(), i));
     }
 
     return result;
@@ -280,12 +272,18 @@ string
 IcePy::PyException::getTypeName()
 {
 #ifdef ICEPY_OLD_EXCEPTIONS
-    PyObject* cls = (PyObject*)((PyInstanceObject*)ex.get())->in_class;
+    PyObject* cls = reinterpret_cast<PyObject*>(reinterpret_cast<PyInstanceObject*>(ex.get())->in_class);
 #else
-    PyObject* cls = (PyObject*)ex.get()->ob_type;
+    PyObject* cls = reinterpret_cast<PyObject*>(ex.get()->ob_type);
 #endif
-    PyObjectHandle str = PyObject_Str(cls);
-    return PyString_AsString(str.get());
+    PyObjectHandle name = PyObject_GetAttrString(cls, "__name__");
+    assert(name.get());
+    PyObjectHandle mod = PyObject_GetAttrString(cls, "__module__");
+    assert(mod.get());
+    string result = PyString_AsString(mod.get());
+    result += ".";
+    result += PyString_AsString(name.get());
+    return result;
 }
 
 IcePy::AllowThreads::AllowThreads()
@@ -313,20 +311,20 @@ IcePy::listToStringSeq(PyObject* l, Ice::StringSeq& seq)
 {
     assert(PyList_Check(l));
 
-    Py_ssize_t sz = PyList_Size(l);
+    Py_ssize_t sz = PyList_GET_SIZE(l);
     for(Py_ssize_t i = 0; i < sz; ++i)
     {
-        PyObject* item = PyList_GetItem(l, i);
-        if(item == NULL)
+        PyObject* item = PyList_GET_ITEM(l, i);
+        if(!item)
         {
             return false;
         }
-        char* str = PyString_AsString(item);
-        if(str == NULL)
+        if(!PyString_Check(item))
         {
+            PyErr_Format(PyExc_ValueError, STRCAST("list element must be a string"));
             return false;
         }
-        seq.push_back(str);
+        seq.push_back(string(PyString_AS_STRING(item), PyString_GET_SIZE(item)));
     }
 
     return true;
@@ -340,7 +338,7 @@ IcePy::stringSeqToList(const Ice::StringSeq& seq, PyObject* l)
     for(Ice::StringSeq::const_iterator p = seq.begin(); p != seq.end(); ++p)
     {
         PyObject* str = Py_BuildValue(STRCAST("s"), p->c_str());
-        if(str == NULL)
+        if(!str)
         {
             Py_DECREF(l);
             return false;
@@ -357,6 +355,30 @@ IcePy::stringSeqToList(const Ice::StringSeq& seq, PyObject* l)
     return true;
 }
 
+bool
+IcePy::tupleToStringSeq(PyObject* t, Ice::StringSeq& seq)
+{
+    assert(PyTuple_Check(t));
+
+    int sz = static_cast<int>(PyTuple_GET_SIZE(t));
+    for(int i = 0; i < sz; ++i)
+    {
+        PyObject* item = PyTuple_GET_ITEM(t, i);
+        if(!item)
+        {
+            return false;
+        }
+        if(!PyString_Check(item))
+        {
+            PyErr_Format(PyExc_ValueError, STRCAST("tuple element must be a string"));
+            return false;
+        }
+        seq.push_back(string(PyString_AS_STRING(item), PyString_GET_SIZE(item)));
+    }
+
+    return true;
+}
+
 bool
 IcePy::dictionaryToContext(PyObject* dict, Ice::Context& context)
 {
@@ -368,13 +390,13 @@ IcePy::dictionaryToContext(PyObject* dict, Ice::Context& context)
     while(PyDict_Next(dict, &pos, &key, &value))
     {
         char* keystr = PyString_AsString(key);
-        if(keystr == NULL)
+        if(!keystr)
         {
             PyErr_Format(PyExc_ValueError, STRCAST("context key must be a string"));
             return false;
         }
         char* valuestr = PyString_AsString(value);
-        if(valuestr == NULL)
+        if(!valuestr)
         {
             PyErr_Format(PyExc_ValueError, STRCAST("context value must be a string"));
             return false;
@@ -394,7 +416,7 @@ IcePy::contextToDictionary(const Ice::Context& ctx, PyObject* dict)
     {
         PyObjectHandle key = PyString_FromString(const_cast<char*>(p->first.c_str()));
         PyObjectHandle value = PyString_FromString(const_cast<char*>(p->second.c_str()));
-        if(key.get() == NULL || value.get() == NULL)
+        if(!key.get() || !value.get())
         {
             return false;
         }
@@ -419,19 +441,19 @@ IcePy::lookupType(const string& typeName)
     // First search for the module in sys.modules.
     //
     PyObject* sysModules = PyImport_GetModuleDict();
-    assert(sysModules != NULL);
+    assert(sysModules);
 
     PyObject* module = PyDict_GetItemString(sysModules, const_cast<char*>(moduleName.c_str()));
     PyObject* dict;
-    if(module == NULL)
+    if(!module)
     {
         //
         // Not found, so we need to import the module.
         //
         PyObjectHandle h = PyImport_ImportModule(const_cast<char*>(moduleName.c_str()));
-        if(h.get() == NULL)
+        if(!h.get())
         {
-            return NULL;
+            return 0;
         }
 
         dict = PyModule_GetDict(h.get());
@@ -441,7 +463,7 @@ IcePy::lookupType(const string& typeName)
         dict = PyModule_GetDict(module);
     }
 
-    assert(dict != NULL);
+    assert(dict);
     return PyDict_GetItemString(dict, const_cast<char*>(name.c_str()));
 }
 
@@ -454,9 +476,9 @@ IcePy::createExceptionInstance(PyObject* type)
     assert(PyExceptionClass_Check(type));
 #endif
     IcePy::PyObjectHandle args = PyTuple_New(0);
-    if(args.get() == NULL)
+    if(!args.get())
     {
-        return NULL;
+        return 0;
     }
     return PyEval_CallObject(type, args.get());
 }
@@ -471,6 +493,37 @@ convertLocalException(const Ice::LocalException& ex, PyObject* p)
     {
         ex.ice_throw();
     }
+    catch(const Ice::InitializationException& e)
+    {
+        IcePy::PyObjectHandle m = PyString_FromString(const_cast<char*>(e.reason.c_str()));
+        PyObject_SetAttrString(p, STRCAST("reason"), m.get());
+    }
+    catch(const Ice::PluginInitializationException& e)
+    {
+        IcePy::PyObjectHandle m = PyString_FromString(const_cast<char*>(e.reason.c_str()));
+        PyObject_SetAttrString(p, STRCAST("reason"), m.get());
+    }
+    catch(const Ice::AlreadyRegisteredException& e)
+    {
+        IcePy::PyObjectHandle m;
+        m = PyString_FromString(const_cast<char*>(e.kindOfObject.c_str()));
+        PyObject_SetAttrString(p, STRCAST("kindOfObject"), m.get());
+        m = PyString_FromString(const_cast<char*>(e.id.c_str()));
+        PyObject_SetAttrString(p, STRCAST("id"), m.get());
+    }
+    catch(const Ice::NotRegisteredException& e)
+    {
+        IcePy::PyObjectHandle m;
+        m = PyString_FromString(const_cast<char*>(e.kindOfObject.c_str()));
+        PyObject_SetAttrString(p, STRCAST("kindOfObject"), m.get());
+        m = PyString_FromString(const_cast<char*>(e.id.c_str()));
+        PyObject_SetAttrString(p, STRCAST("id"), m.get());
+    }
+    catch(const Ice::TwowayOnlyException& e)
+    {
+        IcePy::PyObjectHandle m = PyString_FromString(const_cast<char*>(e.operation.c_str()));
+        PyObject_SetAttrString(p, STRCAST("operation"), m.get());
+    }
     catch(const Ice::UnknownException& e)
     {
         IcePy::PyObjectHandle m = PyString_FromString(const_cast<char*>(e.unknown.c_str()));
@@ -521,7 +574,14 @@ convertLocalException(const Ice::LocalException& ex, PyObject* p)
         m = PyString_FromString(const_cast<char*>(e.operation.c_str()));
         PyObject_SetAttrString(p, STRCAST("operation"), m.get());
     }
-    catch(const Ice::SyscallException& e)
+    catch(const Ice::FileException& e)
+    {
+        IcePy::PyObjectHandle m = PyInt_FromLong(e.error);
+        PyObject_SetAttrString(p, STRCAST("error"), m.get());
+        m = PyString_FromString(const_cast<char*>(e.path.c_str()));
+        PyObject_SetAttrString(p, STRCAST("path"), m.get());
+    }
+    catch(const Ice::SyscallException& e) // This must appear after all subclasses of SyscallException.
     {
         IcePy::PyObjectHandle m = PyInt_FromLong(e.error);
         PyObject_SetAttrString(p, STRCAST("error"), m.get());
@@ -566,37 +626,31 @@ convertLocalException(const Ice::LocalException& ex, PyObject* p)
         m = PyString_FromString(const_cast<char*>(e.type.c_str()));
         PyObject_SetAttrString(p, STRCAST("type"), m.get());
     }
-    catch(const Ice::MarshalException& e)
+    catch(const Ice::UnexpectedObjectException& e)
+    {
+        IcePy::PyObjectHandle m;
+        m = PyString_FromString(const_cast<char*>(e.reason.c_str()));
+        PyObject_SetAttrString(p, STRCAST("reason"), m.get());
+        m = PyString_FromString(const_cast<char*>(e.type.c_str()));
+        PyObject_SetAttrString(p, STRCAST("type"), m.get());
+        m = PyString_FromString(const_cast<char*>(e.expectedType.c_str()));
+        PyObject_SetAttrString(p, STRCAST("expectedType"), m.get());
+    }
+    catch(const Ice::ProtocolException& e) // This must appear after all subclasses of ProtocolException.
     {
         IcePy::PyObjectHandle m = PyString_FromString(const_cast<char*>(e.reason.c_str()));
         PyObject_SetAttrString(p, STRCAST("reason"), m.get());
     }
-    catch(const Ice::PluginInitializationException& e)
+    catch(const Ice::FeatureNotSupportedException& e)
+    {
+        IcePy::PyObjectHandle m = PyString_FromString(const_cast<char*>(e.unsupportedFeature.c_str()));
+        PyObject_SetAttrString(p, STRCAST("unsupportedFeature"), m.get());
+    }
+    catch(const Ice::SecurityException& e)
     {
         IcePy::PyObjectHandle m = PyString_FromString(const_cast<char*>(e.reason.c_str()));
         PyObject_SetAttrString(p, STRCAST("reason"), m.get());
     }
-    catch(const Ice::AlreadyRegisteredException& e)
-    {
-        IcePy::PyObjectHandle m;
-        m = PyString_FromString(const_cast<char*>(e.kindOfObject.c_str()));
-        PyObject_SetAttrString(p, STRCAST("kindOfObject"), m.get());
-        m = PyString_FromString(const_cast<char*>(e.id.c_str()));
-        PyObject_SetAttrString(p, STRCAST("id"), m.get());
-    }
-    catch(const Ice::NotRegisteredException& e)
-    {
-        IcePy::PyObjectHandle m;
-        m = PyString_FromString(const_cast<char*>(e.kindOfObject.c_str()));
-        PyObject_SetAttrString(p, STRCAST("kindOfObject"), m.get());
-        m = PyString_FromString(const_cast<char*>(e.id.c_str()));
-        PyObject_SetAttrString(p, STRCAST("id"), m.get());
-    }
-    catch(const Ice::TwowayOnlyException& e)
-    {
-        IcePy::PyObjectHandle m = PyString_FromString(const_cast<char*>(e.operation.c_str()));
-        PyObject_SetAttrString(p, STRCAST("operation"), m.get());
-    }
     catch(const Ice::LocalException&)
     {
         //
@@ -622,10 +676,10 @@ IcePy::convertException(const Ice::Exception& ex)
     catch(const Ice::LocalException& e)
     {
         type = lookupType(scopedToName(e.ice_name()));
-        if(type != NULL)
+        if(type)
         {
             p = createExceptionInstance(type);
-            if(p.get() != NULL)
+            if(p.get())
             {
                 convertLocalException(e, p.get());
             }
@@ -633,9 +687,9 @@ IcePy::convertException(const Ice::Exception& ex)
         else
         {
             type = lookupType("Ice.UnknownLocalException");
-            assert(type != NULL);
+            assert(type);
             p = createExceptionInstance(type);
-            if(p.get() != NULL)
+            if(p.get())
             {
                 PyObjectHandle s = PyString_FromString(const_cast<char*>(str.c_str()));
                 PyObject_SetAttrString(p.get(), STRCAST("unknown"), s.get());
@@ -645,9 +699,9 @@ IcePy::convertException(const Ice::Exception& ex)
     catch(const Ice::UserException&)
     {
         type = lookupType("Ice.UnknownUserException");
-        assert(type != NULL);
+        assert(type);
         p = createExceptionInstance(type);
-        if(p.get() != NULL)
+        if(p.get())
         {
             PyObjectHandle s = PyString_FromString(const_cast<char*>(str.c_str()));
             PyObject_SetAttrString(p.get(), STRCAST("unknown"), s.get());
@@ -656,9 +710,9 @@ IcePy::convertException(const Ice::Exception& ex)
     catch(const Ice::Exception&)
     {
         type = lookupType("Ice.UnknownException");
-        assert(type != NULL);
+        assert(type);
         p = createExceptionInstance(type);
-        if(p.get() != NULL)
+        if(p.get())
         {
             PyObjectHandle s = PyString_FromString(const_cast<char*>(str.c_str()));
             PyObject_SetAttrString(p.get(), STRCAST("unknown"), s.get());
@@ -672,9 +726,9 @@ void
 IcePy::setPythonException(const Ice::Exception& ex)
 {
     PyObjectHandle p = convertException(ex);
-    if(p.get() != NULL)
+    if(p.get())
     {
-	setPythonException(p.get());
+        setPythonException(p.get());
     }
 }
 
@@ -685,14 +739,14 @@ IcePy::setPythonException(PyObject* ex)
     // PyErr_Restore steals references to the type and exception.
     //
 #ifdef ICEPY_OLD_EXCEPTIONS
-    PyObject* type = (PyObject*)((PyInstanceObject*)ex)->in_class;
+    PyObject* type = reinterpret_cast<PyObject*>(reinterpret_cast<PyInstanceObject*>(ex)->in_class);
     Py_INCREF(type);
 #else
     PyObject* type = PyObject_Type(ex);
-    assert(type != NULL);
+    assert(type);
 #endif
     Py_INCREF(ex);
-    PyErr_Restore(type, ex, NULL);
+    PyErr_Restore(type, ex, 0);
 }
 
 void
@@ -744,15 +798,15 @@ IcePy::createIdentity(const Ice::Identity& ident)
 {
     PyObject* identityType = lookupType("Ice.Identity");
 
-    PyObjectHandle obj = PyObject_CallObject(identityType, NULL);
-    if(obj.get() == NULL)
+    PyObjectHandle obj = PyObject_CallObject(identityType, 0);
+    if(!obj.get())
     {
-        return NULL;
+        return 0;
     }
 
     if(!setIdentity(obj.get(), ident))
     {
-        return NULL;
+        return 0;
     }
 
     return obj.release();
@@ -771,7 +825,7 @@ IcePy::setIdentity(PyObject* p, const Ice::Identity& ident)
     assert(checkIdentity(p));
     PyObjectHandle name = PyString_FromString(const_cast<char*>(ident.name.c_str()));
     PyObjectHandle category = PyString_FromString(const_cast<char*>(ident.category.c_str()));
-    if(name.get() == NULL || category.get() == NULL)
+    if(!name.get() || !category.get())
     {
         return false;
     }
@@ -789,20 +843,20 @@ IcePy::getIdentity(PyObject* p, Ice::Identity& ident)
     assert(checkIdentity(p));
     PyObjectHandle name = PyObject_GetAttrString(p, STRCAST("name"));
     PyObjectHandle category = PyObject_GetAttrString(p, STRCAST("category"));
-    if(name.get() != NULL)
+    if(name.get())
     {
         char* s = PyString_AsString(name.get());
-        if(s == NULL)
+        if(!s)
         {
             PyErr_Format(PyExc_ValueError, STRCAST("identity name must be a string"));
             return false;
         }
         ident.name = s;
     }
-    if(category.get() != NULL)
+    if(category.get())
     {
         char* s = PyString_AsString(category.get());
-        if(s == NULL)
+        if(!s)
         {
             PyErr_Format(PyExc_ValueError, STRCAST("identity category must be a string"));
             return false;
@@ -820,13 +874,13 @@ IcePy_identityToString(PyObject* /*self*/, PyObject* args)
     PyObject* p;
     if(!PyArg_ParseTuple(args, STRCAST("O!"), identityType, &p))
     {
-        return NULL;
+        return 0;
     }
 
     Ice::Identity id;
     if(!IcePy::getIdentity(p, id))
     {
-        return NULL;
+        return 0;
     }
 
     string s;
@@ -837,7 +891,7 @@ IcePy_identityToString(PyObject* /*self*/, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         IcePy::setPythonException(ex);
-        return NULL;
+        return 0;
     }
     return PyString_FromString(const_cast<char*>(s.c_str()));
 }
@@ -849,7 +903,7 @@ IcePy_stringToIdentity(PyObject* /*self*/, PyObject* args)
     char* str;
     if(!PyArg_ParseTuple(args, STRCAST("s"), &str))
     {
-        return NULL;
+        return 0;
     }
 
     Ice::Identity id;
@@ -860,7 +914,7 @@ IcePy_stringToIdentity(PyObject* /*self*/, PyObject* args)
     catch(const Ice::Exception& ex)
     {
         IcePy::setPythonException(ex);
-        return NULL;
+        return 0;
     }
 
     return IcePy::createIdentity(id);
